# -*- coding: utf-8 -*-
# _l1norm.py
# Module providing the l1norm function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the l1norm() function
"""

from __future__ import division

from warnings import warn

import numpy as np
from scipy.signal import dimpulse, lti

from ._utils import _get_zpk, _is_A_B_C_D, _is_num_den, _is_zpk


def l1norm(H):
    """Compute the l1-norm of a z-domain transfer function.

        The norm is evaluated over the first 100 samples.

        **Parameters:**

        H : sequence or lti object
            Any supported LTI representation is accepted.

        **Returns:**

        l1normH : float
            The L1 norm of ``H``.

        .. note:
            LTI objects are translated to ZPK tuples, with possible
            rounding errors.

    """
    if _is_zpk(H):
        z, p, k = H
        HP = (z, p, k, 1.)
    elif _is_num_den(H):
        num, den = H
        HP = (num, den, 1.)
    elif _is_A_B_C_D(H):
        A, B, C, D = H
        HP = (A, B, C, D, 1.)
    elif isinstance(H, lti):
        warn('l1norm() got an LTI object, translated to zpk form, rounding errors possible.')
        z, p, k = _get_zpk(H)
        HP = (z, p, k, 1.)
    _, y = dimpulse(HP, t=np.arange(100))
    return np.sum(np.abs(y[0]))



# -*- coding: utf-8 -*-
# test_SIunits.py
# This module provides the tests for the SIunits function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the SIunits() function.
"""

import unittest
import numpy as np

from deltasigma import SIunits

class TestSIunits(unittest.TestCase):
    """Test class for SIunits()"""

    def setUp(self):
        pass

    def test_SIunits(self):
        """Test function for SIunits() 1/2"""
        tv = (0, 1, 1e3, 2100312.24, .32545, 21e-9, 34e-12, 9569300e-12)
        correct = (0, ''), (1, ''), (1e3, 'k'), (1e6, 'M'), (1e-3, 'm'), \
                  (1e-9, 'n'), (1e-12, 'p'), (1e-6, 'u')
        f, p = SIunits(tv)
        res = zip(f,p)
        for r, c in zip(res, correct):
            self.assertTrue(r[0] == c[0] and r[1] == c[1])

    def test_SIunits_2(self):
        """Test function for SIunits() 2/2"""
        # test scalars
        tv = 2100312.24
        correct = (1e6, 'M')
        f, p = SIunits(tv)
        self.assertTrue(f == correct[0] and p == correct[1])




# -*- coding: utf-8 -*-
# _calculateTF.py
# Module providing the calculateTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the calculateTF() function
"""

import numpy as np
from scipy.signal import lti, ss2zpk

from ._constants import eps
from ._partitionABCD import partitionABCD
from ._utils import minreal


def calculateTF(ABCD, k=1.):
    """Calculate the NTF and STF of a delta-sigma modulator.

    The calculation is performed for a given loop filter
    ABCD matrix, assuming a quantizer gain of ``k``.

    **Parameters:**

    ABCD : array_like,
        The ABCD matrix that describes the system.

    k : float, optional
        The quantizer gain. If not specified, a default value of 1 is used.

    **Returns:**

    (NTF, STF) : a tuple of two LTI objects.

    **Example:**

    Realize a fifth-order modulator with the cascade-of-resonators structure,
    feedback form. Calculate the ABCD matrix of the loop filter and verify
    that the NTF and STF are correct.

    .. code-block:: python

        from deltasigma import *
        H = synthesizeNTF(5, 32, 1)
        a, g, b, c = realizeNTF(H)
        ABCD = stuffABCD(a,g,b,c)
        ntf, stf = calculateTF(ABCD)

    From which we get:

    ``H``::

             (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)
        --------------------------------------------------------
         (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665)

    coefficients::

        a: 0.0007, 0.0084, 0.055, 0.2443, 0.5579
        g: 0.0028, 0.0079
        b: 0.0007, 0.0084, 0.055, 0.2443, 0.5579, 1.0
        c: 1.0, 1.0, 1.0, 1.0, 1.0

    ABCD matrix::

        [[  1.00000000e+00   0.00000000e+00   0.00000000e+00   0.00000000e+00
            0.00000000e+00   6.75559806e-04  -6.75559806e-04]
         [  1.00000000e+00   1.00000000e+00  -2.79396240e-03   0.00000000e+00
            0.00000000e+00   8.37752565e-03  -8.37752565e-03]
         [  1.00000000e+00   1.00000000e+00   9.97206038e-01   0.00000000e+00
            0.00000000e+00   6.33294166e-02  -6.33294166e-02]
         [  0.00000000e+00   0.00000000e+00   1.00000000e+00   1.00000000e+00
           -7.90937431e-03   2.44344030e-01  -2.44344030e-01]
         [  0.00000000e+00   0.00000000e+00   1.00000000e+00   1.00000000e+00
            9.92090626e-01   8.02273699e-01  -8.02273699e-01]
         [  0.00000000e+00   0.00000000e+00   0.00000000e+00   0.00000000e+00
            1.00000000e+00   1.00000000e+00   0.00000000e+00]]

    NTF::

             (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)
        --------------------------------------------------------
         (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665)

    STF::

        1

    """
    A, B, C, D = partitionABCD(ABCD)
    if B.shape[1] > 1:
        B1 = B[:, 0]
        B2 = B[:, 1]
        B1 = B1.reshape((B1.shape[0], 1)) if len(B1.shape) == 1 else B1
        B2 = B2.reshape((B2.shape[0], 1)) if len(B2.shape) == 1 else B2
    else:
        B1 = B
        B2 = B

    # Find the noise transfer function by forming the closed-loop
    # system (sys_cl) in state-space form.
    Acl = A + k * np.dot(B2, C)
    Bcl = np.hstack((B1 + k*B2*D[0, 0], B2))
    Ccl = k*C
    Dcl = np.array((k*D[0, 0], 1.))
    Dcl = Dcl.reshape((1, Dcl.shape[0])) if len(Dcl.shape) == 1 else Dcl
    tol = min(1e-3, max(1e-6, eps**(1/ABCD.shape[0])))
    # input #0 is the signal
    # input #1 is the quantization noise
    stf_p, stf_z, stf_k  = ss2zpk(Acl, Bcl, Ccl, Dcl, input=0)
    ntf_p, ntf_z, ntf_k = ss2zpk(Acl, Bcl, Ccl, Dcl, input=1)
    stf = lti(stf_p, stf_z, stf_k)
    ntf = lti(ntf_p, ntf_z, ntf_k)
    stf_min, ntf_min = minreal((stf, ntf), tol)
    return ntf_min, stf_min



import unittest
import numpy as np
import deltasigma as ds

from nose.tools import raises

class TestAxisLabels(unittest.TestCase):
    """Test function for axisLabels()"""
    def setUp(self):
        self.ran = np.arange(100)
    
    def test_axis_label_geneation_part1(self):
        """Test function for axisLabels() 1/3"""
        ss = ds.axisLabels(self.ran, incr=10)
        r = ['0', '10', '20', '30', '40', '50', '60', '70', '80', '90']
        self.assertTrue(r == ss)
        
    def test_axis_label_generation_part2(self):
        """Test function for axisLabels() 2/3"""
        ss = ds.axisLabels(self.ran, incr=(15, 10))
        r = ['10', '25', '40', '55', '70', '85']
        self.assertTrue(r == ss)

    @raises(ValueError)
    def test_axis_label_incr_length(self):
        """Test function for axisLabels() 3/3"""
        ds.axisLabels([1., 2., 3., 4.], [1, 2, 3])




# -*- coding: utf-8 -*-
# test_mapQtoR.py
# This module provides the tests for the mapQtoR function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the mapQtoR() function.
"""

from __future__ import division

import unittest
import numpy as np
import deltasigma as ds

class TestMapQtoR(unittest.TestCase):
    """Test class for mapQtoR()"""

    def setUp(self):
        A = np.arange(1, 6*7 + 1, dtype=np.int16).reshape((7, 6)).T
        self.A = A + 1j*A
        self.Ares = \
        np.array([[1, -1, 7, -7, 13, -13, 19, -19, 25, -25, 31, -31, 37, -37],
                  [1, 1, 7, 7, 13, 13, 19, 19, 25, 25, 31, 31, 37, 37],
                  [2, -2, 8, -8, 14, -14, 20, -20, 26, -26, 32, -32, 38, -38],
                  [2, 2, 8, 8, 14, 14, 20, 20, 26, 26, 32, 32, 38, 38],
                  [3, -3, 9, -9, 15, -15, 21, -21, 27, -27, 33, -33, 39, -39],
                  [3, 3, 9, 9, 15, 15, 21, 21, 27, 27, 33, 33, 39, 39],
                  [4, -4, 10, -10, 16, -16, 22, -22, 28, -28, 34, -34, 40, -40],
                  [4, 4, 10, 10, 16, 16, 22, 22, 28, 28, 34, 34, 40, 40],
                  [5, -5, 11, -11, 17, -17, 23, -23, 29, -29, 35, -35, 41, -41],
                  [5, 5, 11, 11, 17, 17, 23, 23, 29, 29, 35, 35, 41, 41],
                  [6, -6, 12, -12, 18, -18, 24, -24, 30, -30, 36, -36, 42, -42],
                  [6, 6, 12, 12, 18, 18, 24, 24, 30, 30, 36, 36, 42, 42]],
                 dtype=np.int16)

    def test_mapQtoR(self):
        """Test function for mapQtoR()"""
        At = ds.mapQtoR(self.A)
        self.assertTrue(np.allclose(At, self.Ares))




# -*- coding: utf-8 -*-
# _bunquantize.py
# Module providing the bunquantize function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the bunquantize() function
"""

import numpy as np

from ._utils import carray


def bunquantize(q):
    """The value corresponding to a bidirectionally quantized quantity.

    ``q`` is a (2n, m) ndarray containing the powers of 2 and their signs
    for each quantized value.

    .. seealso::
        :func:`bquantize`

    """
    y = []
    for qi in q:
        y += [(qi[1:qi.shape[0]+1:2, :]*2.**qi[:qi.shape[0]:2, :]).sum()]
    return carray(y)



# -*- coding: utf-8 -*-
# test_ds_optzeros.py
# This module provides the tests for the ds_freq() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds
import pkg_resources
import scipy.io
from os.path import join


class TestDSOptZeros(unittest.TestCase):
    """Test function for ds_optzeros()"""

    def setUp(self):
        file_path = join('test_data', 'test_ds_optzeros.mat')
        fname = pkg_resources.resource_filename(__name__, file_path)
        self.res = scipy.io.loadmat(fname)['res']

    def test_opt_zeros(self):
        """ Test 14 different optzeros() calls. """
        ns = ('n1', 'n2', 'n3', 'n4', 'n5', 'n6', 'n7', 'n8', 'n9', 'n10',
              'n11', 'n12', 'n13', 'n14')

        for i, n in enumerate(ns):
            for opt in (0, 1, 2):
                self.assertTrue(
                    np.allclose(
                        self.res[n][0][0][:, opt], ds.ds_optzeros(i+1, opt),
                        atol=1e-10, rtol=1e-6))



# -*- coding: utf-8 -*-
# test_nabsH.py
# This module provides the tests for the nabsH function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the nabsH() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from nose.tools import raises

class TestNabsH(unittest.TestCase):
    """Test class for nabsH()"""

    def setUp(self):
        pass

    def test_nabsH(self):
        """Test function for nabsH()"""
        H = ([1, 2], [2, 0, .25], 1)
        N = 129
        w = np.linspace(0, 2*np.pi, num=N, endpoint=True)
        z = np.exp(1j*w)
        r1 = -np.abs(ds.evalTF(H, z))
        r2 = ds.nabsH(w, H)
        self.assertTrue(np.allclose(r1, r2, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# test_rmsGain.py
# This module provides the tests for the rmsGain function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the rmsGain() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestRmsGain(unittest.TestCase):
    """Test class for rmsGain()"""

    def setUp(self):
        num = (1,)
        den = (1, 2, 10)
        self.H = (num, den)
        self.f1 = 0.001
        self.f2 = 0.5
        self.res1 = 0.102245275091

    def test_rmsGain(self):
        """Test function for rmsGain()"""
        res = ds.rmsGain(self.H, self.f1, self.f2, N=1000)
        self.assertTrue(np.allclose((res,), (self.res1,), rtol=1e-05,
                                    atol=1e-08))




# -*- coding: utf-8 -*-
# _mapRtoQ.py
# Module providing the mapRtoQ function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the mapRtoQ() function
"""

from __future__ import division

import numpy as np


def mapRtoQ(ABCDr):
    """Map a real ABCD matrix to a quadrature one.

    **Parameters:**

    ABCDr : ndarray
        A real matrix describing a quadrature system.

    ``ABCDr`` has its states paired (real, imaginary).

    **Returns:**

    (ABCDq, ABCDp) : tuple

    Where:

    ABCDq : ndarray
        is the quadrature (complex) version of ABCDr.
    ABCDp : ndarray
        is the mirror-image system matrix.

    .. note:: ``ABCDp`` is zero if ``ABCDr`` has no quadrature errors.
    """
    ABCD11 = ABCDr[::2,   ::2]
    ABCD12 = ABCDr[::2,  1::2]
    ABCD21 = ABCDr[1::2,  ::2]
    ABCD22 = ABCDr[1::2, 1::2]

    ABCDq = 0.5*(ABCD11 + ABCD22) + 0.5j*(ABCD21 - ABCD12);
    ABCDp = 0.5*(ABCD11 - ABCD22) + 0.5j*(ABCD21 + ABCD12);

    return ABCDq, ABCDp



# -*- coding: utf-8 -*-
# _mapQtoR.py
# Module providing the mapQtoR function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the mapQtoR() function
"""

from __future__ import division

import numpy as np


def mapQtoR(ABCD):
    """Map a quadrature ABCD matrix to a real one.

    Each element z in ABCD is replaced by a 2x2 matrix in ``ABCDr``, the
    return value.

    Specifically:

    .. math::

        z \\rightarrow
         \\begin{bmatrix}
          x & -y \\\\
          y & x \\\\
         \\end{bmatrix}
        \\mathrm{where}\\ x = Re(z)\\ \\mathrm{and}\\ y = Im(z)

    The non-quadrature topology can be simulated with :func:`simulateDSM`.::

        import numpy as np
        from deltasigma import *
        nlev = 9
        f0 = 1./16.
        osr = 32
        M = nlev - 1
        ntf = synthesizeQNTF(4, osr, f0, -50, -10)
        N = 64*osr
        f = int(np.round((f0 + 0.3*0.5/osr)*N)/N)
        u = 0.5*M*np.exp(2j*np.pi*f*np.arange(N))
        # Instead of calling simulateQDSM
        # v = simulateQDSM(u, ntf, nlev)
        # it's faster to run:
        ABCD = realizeQNTF(ntf, 'FF')
        ABCDr = mapQtoR(ABCD)
        ur = np.vstack((np.real(u), np.imag(u)))
        vr = simulateDSM(ur, ABCDr, nlev*np.array([[1],[1]]))
        v = vr[0,:] + 1j*vr[1, :]

    Notice the example above requires the function :func:`synthesizeQNTF`,
    which is not part of the current release of python-deltasigma.

    """
    A = np.zeros((2*ABCD.shape[0], 2*ABCD.shape[1]))
    A[::2, ::2] = np.real(ABCD)
    A[1::2, 1::2] = np.real(ABCD)
    A[::2, 1::2] = -np.imag(ABCD)
    A[1::2, ::2] = +np.imag(ABCD)
    return A



# -*- coding: utf-8 -*-
# _stuffABCD.py
# Module providing the stuffABCD function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the stuffABCD() function
"""

from __future__ import division, print_function

import numpy as np

from ._partitionABCD import partitionABCD
from ._utils import carray, diagonal_indices


def stuffABCD(a, g, b, c, form='CRFB'):
    """Calculate the ABCD matrix from the parameters of a modulator topology.

    **Parameters:**

    a : array_like
        Feedback/feedforward coefficients from/to the quantizer. Length :math:`n`.
    g : array_like
        Resonator coefficients. Length :math:`floor(n/2)`.
    b : array_like
        Feed-in coefficients from the modulator input to each integrator. Length :math:`n + 1`.
    c : array_like
        Integrator inter-stage coefficients. Length :math:`n`.
    form : str, optional
        See :func:`realizeNTF` for a list of supported structures.

    **Returns:**

    ABCD : ndarray
        A state-space description of the modulator loop filter.
    
    .. seealso:: :func:`mapABCD`, the inverse function.
    """
    # Code common to all structures.
    # a, g, b, and c are internally row vectors
    a = carray(a).reshape((1, -1))
    g = carray(g).reshape((1, -1))
    b = carray(b).reshape((1, -1))
    c = carray(c).reshape((1, -1))
    order = max(a.shape)
    odd = order % 2
    even = 1 - odd # OMG
    ABCD = np.zeros((order + 1, order + 2))
    if np.isscalar(b) or max(b.shape) == 1:
        b = np.atleast_2d(b)
        b = np.hstack((b, np.zeros((1, order))))

    # mutually exclusive matrix stuffing
    if form == 'CRFB':
        # C=(0 0...c_n)
        # This is done as part of the construction of A, below
        # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b
        # B2 = -(a_1 a_2... a_n)
        ABCD[:order, order + 1] = -a
        diagonal = diagonal_indices(ABCD[:order, :order])
        ABCD[diagonal] = np.ones((order,))
        subdiag = [i[even:order:2] for i in diagonal_indices(ABCD, -1)]
        ABCD[subdiag] = c[0, even:order:2]
        if order > odd:
            supdiag = [i[odd:order:2] for i in diagonal_indices(ABCD[:order, :order], +1)]
            ABCD[supdiag] = -g.reshape((-1,))
        # row numbers of delaying integrators
        dly = np.arange(odd + 1, order, 2)
        ABCD[dly, :] = ABCD[dly, :] + np.dot(np.diag(c[0, dly - 1]), ABCD[dly - 1, :])
    elif form == 'CRFF':
        # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b
        # B2 = -(c_1 0... 0)
        ABCD[0, order + 1] = -c[0, 0]
        # number of elements = order
        diagonal = diagonal_indices(ABCD[:order, :order])
        ABCD[diagonal] = np.ones((order, ))
        # subdiag = diagonal[:order - 1:2] + 1
        subdiag = [i[:order - 1:2] for i in diagonal_indices(ABCD, -1)]
        ABCD[subdiag] = c[0, 1:order:2]
        if even:
            # rows to have g*(following row) subtracted.
            multg = np.arange(0, order, 2)
            ABCD[multg, :] = ABCD[multg, :] \
                             - np.dot(np.diag(g.reshape((-1,))), ABCD[multg + 1, :])
        else:
            if order >= 3:
                # supdiag = diagonal[2:order:2] - 1
                supdiag = [i[1:order:2] for i in
                           diagonal_indices(ABCD[:order, :order], +1)]
                ABCD[supdiag] = -g.reshape((-1,))
        # rows to have c*(preceding row) added.
        multc = np.arange(2, order, 2)
        ABCD[multc, :] = ABCD[multc, :] + np.dot(np.diag(c[0, multc]), ABCD[multc - 1, :])
        ABCD[order, :order:2] = a[0, :order:2]
        for i in range(1, order, 2):
            ABCD[order, :] = ABCD[order, :] + a[0, i]*ABCD[i, :]
    elif form == 'CIFB':
        # C=(0 0...c_n)
        # This is done as part of the construction of A, below
        # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b
        # B2 = -(a_1 a_2... a_n)
        ABCD[:order, order + 1] = -a
        diagonal = diagonal_indices(ABCD[:order, :order])
        ABCD[diagonal] = np.ones((order,))
        subdiag = diagonal_indices(ABCD, -1)
        ABCD[subdiag] = c.reshape((-1,))
        supdiag = [i[odd:order+odd:2] for i in diagonal_indices(ABCD[:order, :order], +1)]
        ABCD[supdiag] = -g.reshape((-1,))
    elif form == 'CIFF':
        # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b
        # B2 = -(c_1 0... 0)
        ABCD[0, order + 1] = -c[0, 0]
        diagonal = diagonal_indices(ABCD[:order, :order])
        ABCD[diagonal] = np.ones((order, ))
        subdiag = diagonal_indices(ABCD[:order, :order], -1)
        ABCD[subdiag] = c[0, 1:]
        # C = (a_1 a_2... a_n)
        ABCD[order, :order] = a[0, :order]
        # supdiag = diagonal[odd + 1:order:2] - 1
        supdiag = [i[odd:order+odd:2] for i in  
                   diagonal_indices(ABCD[:order, :order], +1)]
        ABCD[supdiag] = -g.reshape((-1,))
    elif form == 'CRFBD':
        # C=(0 0...c_n)
        ABCD[order, order - 1] = c[0, order - 1]
        # B1 = (b_1 b_2... b_n), D=(b_n+1 0)
        ABCD[:, order] = b
        # B2 = -(a_1 a_2... a_n)
        ABCD[:order, order + 1] = -a
        diagonal = diagonal_indices(ABCD[:order, :order])
        ABCD[diagonal] = np.ones((order, ))
        # row numbers of delaying integrators
        dly = np.arange(odd, order, 2)
        subdiag = [np.atleast_1d(i)[dly] for i in
                   diagonal_indices(ABCD[:order, :order], -1)]
        ABCD[subdiag] = c[0, dly]
        supdiag = [np.atleast_1d(i)[dly] for i in 
                   diagonal_indices(ABCD[:order, :order], +1)]
        supdiag = [np.atleast_1d(i[odd:]) for i in supdiag]
        ABCD[dly, :] = ABCD[dly, :] - np.dot(np.diag(g.reshape((-1))), \
                                             ABCD[dly + 1, :])
        if order > 2:
            coupl = np.arange(even + 1, order, 2)
            ABCD[coupl, :] = ABCD[coupl, :] + np.dot(np.diag(c[0, coupl - 1]), \
                             ABCD[coupl - 1, :])
    elif form == 'CRFFD':
        diagonal = diagonal_indices(ABCD[:order, :order])
        subdiag = diagonal_indices(ABCD[:order, :order], -1)
        supdiag = diagonal_indices(ABCD[:order, :order], +1)
        # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b.reshape((-1,))
        # B2 = -(c_1 0... 0)
        ABCD[0, order + 1] = -c[0, 0]
        ABCD[diagonal] = np.ones((order, ))
        # rows to have c*(preceding row) added.
        multc = np.arange(1, order, 2)
        if order > 2:
            ABCD[[i[1::2] for i in subdiag]] = c[0, 2::2]
        if even:
            ABCD[[i[::2] for i in supdiag]] = -g.reshape((-1,))
        else:
            # subtract g*(following row) from the multc rows
            ABCD[multc, :] = ABCD[multc, :] - np.dot(np.diag(g[0, :]), ABCD[multc + 1, :])
        ABCD[multc, :] = ABCD[multc, :] + np.dot(np.diag(c[0, multc]), ABCD[multc - 1, :])
        # C
        ABCD[order, 1:order:2] = a[0, 1:order:2]
        for i in range(0, order, 2):
            ABCD[order, :] = ABCD[order, :] + a[0, i]*ABCD[i, :]
        # The above gives y(n+1); need to add a delay to get y(n).
        # Do this by augmenting the states. Note: this means that
        # the apparent order of the NTF is one higher than it actually is.
        A, B, C, D = partitionABCD(ABCD, 2)
        A = np.vstack((np.hstack((A, np.zeros((order, 1)))),
                       np.hstack((C, np.zeros((1, 1))))))
        B = np.vstack((B, D))
        C = np.hstack((np.zeros((1, order)), np.ones((1, 1))))
        D = np.array([0, 0]).reshape(1, 2)
        ABCD = np.vstack((np.hstack((A, B)), np.hstack((C, D))))
    elif 'PFF' == form:
        # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b.T
        odd_1 = odd     # !! Bold assumption !!
        odd_2 = 0.      # !! Bold assumption !! 
        gc = g.dot(c[odd_1::2])
        theta = np.arccos(1 - gc/2.)
        if odd_1:
            theta0 = 0.
        else:
            theta0 = theta[0]
        order_2 = 2*max(np.flatnonzero(np.abs(theta - theta0) > 0.5).shape)
        order_1 = order - order_2
        # B2 = -(c_1 0...0 c_n 0...0)
        ABCD[0, order + 1] = -c[0]
        ABCD[order_1, order + 1] = -c[order_1]
        # number of elements = order
        diagonal = np.arange(0, order*(order + 1), order + 2)
        ABCD[diagonal] = np.ones((1, order))
        i = np.hstack((np.arange(0, order_1 - 1, 2),
                       np.arange(order - order_2, order, 2)))
        subdiag = diagonal[i] + 1
        ABCD[subdiag] = c[i + 1]
        if odd_1:
            if order_1 >= 3:
                supdiag = diagonal[2:order_1:2] - 1
                ABCD[supdiag] = -g[:(order_1 - 1)/2.]
        else:
            # rows to have g*(following row) subtracted.
            multg = np.arange(0, order_1, 2)
            ABCD[multg, :] = ABCD[multg, :] - np.diag(g[:order_1/2.]) * \
                             ABCD[multg + 1, :]
        if odd_2:
            if order_2 >= 3:
                supdiag = diagonal[order_1 + 1:order:2] - 1
                ABCD[supdiag] = -g[:(order_1 - 1)/2.]
        else:
            # rows to have g*(following row) subtracted.
            multg = np.arange(order_1, order, 2)
            gg = g[(order_1 - odd_1)/2:]
            ABCD[multg, :] = ABCD[multg, :] - np.diag(gg)*ABCD[multg + 1, :]
        # Rows to have c*(preceding row) added.
        multc = np.hstack((np.arange(2, order_1, 2), 
                           np.arange(order_1 + 2, order, 2)
                           )).reshape(1, -1)
        ABCD[multc, :] = ABCD[multc, :] + np.diag(c[multc])*ABCD[multc - 1, :]
        # C portion of ABCD
        i = np.hstack((
                       np.arange(0, order_1, 2),
                       np.arange(order_1, order, 2)
                       )).reshape(1,-1)
        ABCD[order, i] = a[i]
        for i in np.hstack((np.arange(1, order_1, 2),
                            np.arange(order_1 + 1, order, 2)
                            )):
            ABCD[order, :] = ABCD[order, :] + a[i] * ABCD[i, :]
    elif 'Stratos' == form:
        # code copied from case 'CIFF':
        # # B1 = (b_1 b_2... b_n), D=(b_(n+1) 0)
        ABCD[:, order] = b
        # # B2 = -(c_1 0... 0)
        ABCD[0, order + 1] = -c[0, 0]
        diagonal = diagonal_indices(ABCD[:order, :order])
        ABCD[diagonal] = np.ones((order, ))
        subdiag = diagonal_indices(ABCD[:order, :order], -1)
        ABCD[subdiag] = c[0, 1:]
        # code based on case 'CRFF':
        # # rows to have g*(following row) subtracted.
        multg = np.arange(odd, order - 1, 2)
        ABCD[multg, :] = ABCD[multg, :] - np.dot(np.diag(g.reshape((-1,))), \
                                                 ABCD[multg + 1, :])
        # code copied from case 'CIFF':
        # # C = (a_1 a_2... a_n)
        ABCD[order, :order] = a[0, :order]
    else:
        raise ValueError('Form %s is not yet supported.', form)
    return ABCD



# -*- coding: utf-8 -*-
# test_logsmooth.py
# This module provides the tests for the logsmooth function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the logsmooth() function.
"""

from __future__ import division

import unittest
import numpy as np
import pkg_resources, scipy.io

from numpy.fft import fft

from deltasigma import ds_f1f2
from deltasigma import ds_hann
from deltasigma import logsmooth
from deltasigma import simulateDSM
from deltasigma import synthesizeNTF
from deltasigma import undbv

class TestLogsmooth(unittest.TestCase):
    """Test class for logsmooth()"""

    def setUp(self):
        pass

    def test_logsmooth(self):
        """Test function for logsmooth()"""
        f0 = 1./8
        OSR = 64
        order = 8
        N = 8192
        H = synthesizeNTF(order, OSR, 1, 1.5, f0)
        # fB = int(np.ceil(N/(2. * OSR)))
        quadrature = False
        M = 1
        f1, f2 = ds_f1f2(OSR, f0, quadrature)
        Amp = undbv(-3)
        f = .3
        fin = np.round(((1 - f)/2*f1 + (f + 1)/2 * f2) * N)
        t = np.arange(0, N).reshape((1, -1))
        u = Amp * M * np.cos((2*np.pi/N)*fin*t)
        v, _, _, _ = simulateDSM(u, H, M + 1)

        window = ds_hann(N)
        # NBW = 1.5/N
        spec0 = fft(v * window) / (M*N/4)

        ### -1 is really important THIS IS NOT MATLAB!!
        fl, pl = logsmooth(spec0, fin - 1)

        fname = pkg_resources.resource_filename(__name__,
                                                "test_data/test_logsmooth.mat")
        data = scipy.io.loadmat(fname)
        self.assertTrue(np.allclose(fl, data['fl'], atol=1e-8, rtol=1e-5))
        self.assertTrue(np.allclose(pl, data['pl'], atol=1e-8, rtol=1e-5))





# -*- coding: utf-8 -*-
# _evalMixedTF.py
# Module providing the evalMixedTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the evalMixedTF() function
"""

from __future__ import division

import numpy as np

from ._evalTFP import evalTFP
from ._utils import carray, restore_input_form, save_input_form


def evalMixedTF(tf, f, df=1e-5):
    """Compute a mixed transfer function.

    Mathematically, it means to evaluate the sum of products:

    .. math::

        TF(f) = \\sum_i H_{z,i}(f) \\cdot H_{s,i}(f)

    **Parameters:**

    tf : dict
        ``tf`` is a dictionary of lists of 1d arrays, with keys 'Hs' and 'Hz',
        which represent continuous-time and discrete-time TFs which will be
        evaluated, multiplied together and then added up.
    f : scalar or sequence-like
        The frequencies (or frequency, if ``f`` is a scalar) at which the
        product will be evaluated.
    df : float, optional
        If the method happens to evaluate a transfer function in a root, it
        will move away of ``df``, defaulting to 1E-5.

    **Returns:**

    TF : scalar or sequence
        The sum of products computed at ``f``.

    """
    iform = save_input_form(f)
    f = carray(f)
    H = np.zeros(f.shape)
    for i in range(max(len(tf['Hs']), len(tf['Hz']))):
        H = H + evalTFP(tf['Hs'][i], tf['Hz'][i], f)
    err = np.logical_or(np.isnan(H), np.isinf(H))
    if np.any(err):
        for i in np.where(err):
            H[i] = evalMixedTF(tf, f[i] + df, df*10)
    return restore_input_form(H, iform)



# -*- coding: utf-8 -*-
# test_delay.py
# This module provides the tests for the delay function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the delay() function.
"""

import unittest
import numpy as np

from deltasigma import delay

class TestDelay(unittest.TestCase):
    """Test class for delay()"""

    def setUp(self):
        pass

    def test_delay_1(self):
        """Test function for delay() 1/6"""
        v1 = np.arange(4)
        r1 = np.array((0, 0, 0, 0))
        self.assertTrue(np.allclose(delay(v1, 5), r1))

    def test_delay_2(self):
        """Test function for delay() 2/6"""
        v2 = np.ones((10, 1))
        r2 = np.zeros((10, 1))
        r2[5:, 0] = 1
        self.assertTrue(np.allclose(delay(v2, 5), r2))

    def test_delay_3(self):
        """Test function for delay() 3/6"""
        v3 = np.ones((1, 10))
        r3 = np.zeros((1, 10))
        r3[0, 5:] = 1
        self.assertTrue(np.allclose(delay(v3, 5), r3))

    def test_delay_4(self):
        """Test function for delay() 4/6"""
        v4 = np.array([1., 2., 3., 4.])
        r4 = np.array([0., 1., 2., 3.])
        self.assertTrue(np.allclose(delay(v4, 1), r4))

    def test_delay_5(self):
        """Test function for delay() 5/6"""
        v5 = np.array([[1., 2., 3., 4.],
                       [1., 2., 3., 4.]])
        x5 = delay(v5, 1)
        r5 = np.array([[ 0.,  1.,  2.,  3.],
                       [ 0.,  1.,  2.,  3.]])
        self.assertTrue(np.allclose(x5, r5))

    def test_delay_6(self):
        """Test function for delay() 6/6"""
        v5 = np.array([[1., 2., 3., 4.],
                       [1., 2., 3., 4.]])
        v6 = v5.T
        x6 = delay(v6, 1)
        r5 = np.array([[ 0.,  1.,  2.,  3.],
                       [ 0.,  1.,  2.,  3.]])
        r6 = r5.T
        self.assertTrue(np.allclose(x6, r6))



# -*- coding: utf-8 -*-
# _dbv.py
# This module provides the dbv function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the dbv() function, used to convert a voltage gain to dB.
"""

import numpy as np

from ._undbv import undbv
from ._utils import carray, restore_input_form, save_input_form


def dbv(x):
    """Calculate the dB equivalent of the voltage ratio ``x``.

    .. math::

        G_{dB} = 20 \\mathrm{log}_{10}(|x|)

    **Parameters:**

    x : scalar or sequence
        The voltage (ratio) to be converted.

    **Returns:**

    GdB : scalar or sequence
        The input voltage (ratio) expressed in dB.

    .. seealso:: :func:`undbv`, :func:`db`, :func:`dbp`, :func:`dbm`

    """
    iform = save_input_form(x)
    x = carray(x)
    y = -np.inf*np.ones(x.shape)
    nonzero = (x != 0)
    y[nonzero] = 20.*np.log10(np.abs(x[nonzero]))
    return restore_input_form(y, iform)



# -*- coding: utf-8 -*-
# _simulateDSM_python.py
# Module providing the CPython simulateDSM function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the simulateDSM() function
"""

from __future__ import division

import collections
from warnings import warn

import numpy as np
from scipy.linalg import inv, norm, orth
from scipy.signal import tf2zpk, zpk2ss

from ._utils import _get_zpk, carray


def simulateDSM(u, arg2, nlev=2, x0=0.):
    """Simulate a Delta Sigma modulator

    **Syntax:**

     * [v, xn, xmax, y] = simulateDSM(u, ABCD, nlev=2, x0=0)
     * [v, xn, xmax, y] = simulateDSM(u, ntf, nlev=2, x0=0)

    Compute the output of a general delta-sigma modulator with input u,
    a structure described by ABCD, an initial state x0 (default zero) and
    a quantizer with a number of levels specified by nlev.
    Multiple quantizers are implied by making nlev an array,
    and multiple inputs are implied by the number of rows in u.

    Alternatively, the modulator may be described by an NTF.
    The NTF is zpk object. (And the STF is assumed to be 1.)
    The structure that is simulated is the block-diagonal structure used by
    ``zpk2ss()``.

    **Example:**

    Simulate a 5th-order binary modulator with a half-scale sine-wave input and
    plot its output in the time and frequency domains.::

        import numpy as np
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        N = 8192 
        fB = np.ceil(N/(2*OSR))
        f = 85
        u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
        v = simulateDSM(u, H)[0]

    Graphical display of the results:

    .. plot::

        import numpy as np
        import pylab as plt
        from numpy.fft import fft
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        N = 8192 
        fB = np.ceil(N/(2*OSR))
        f = 85
        u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
        v = simulateDSM(u, H)[0]
        plt.figure(figsize=(10, 7))
        plt.subplot(2, 1, 1)
        t = np.arange(85)
        # the equivalent of MATLAB 'stairs' is step in matplotlib
        plt.step(t, u[t], 'g', label='u(n)')
        plt.hold(True)
        plt.step(t, v[t], 'b', label='v(n)')
        plt.axis([0, 85, -1.2, 1.2]);
        plt.ylabel('u, v');
        plt.xlabel('sample')
        plt.legend()
        plt.subplot(2, 1, 2)
        spec = fft(v*ds_hann(N))/(N/4)
        plt.plot(np.linspace(0, 0.5, N/2 + 1), dbv(spec[:N/2 + 1]))
        plt.axis([0, 0.5, -120, 0])
        plt.grid(True)
        plt.ylabel('dBFS/NBW')
        snr = calculateSNR(spec[:fB], f)
        s = 'SNR = %4.1fdB' % snr
        plt.text(0.25, -90, s)
        s =  'NBW = %7.5f' % (1.5/N)
        plt.text(0.25, -110, s)
        plt.xlabel("frequency $1 \\\\rightarrow f_s$")

    Click on "Source" above to see the source code.
    """

    #fprintf(1,'Warning: You are running the non-mex version of simulateDSM.\n');
    #fprintf(1,'Please compile the mex version with "mex simulateDSM.c"\n');

    nlev = carray(nlev)
    u = np.array(u) if not hasattr(u, 'ndim') else u
    if not max(u.shape) == np.prod(u.shape):
        warn("Multiple input delta sigma structures have had little testing.")
    if u.ndim == 1:
        u = u.reshape((1, -1))
    nu = u.shape[0]
    nq = 1 if np.isscalar(nlev) else nlev.shape[0]
    # extract poles and zeros
    if (hasattr(arg2, 'inputs') and not arg2.inputs == 1) or \
       (hasattr(arg2, 'outputs') and not arg2.outputs == 1):
            raise TypeError("The supplied TF isn't a SISO transfer function.")
    if isinstance(arg2, np.ndarray):
        ABCD = np.asarray(arg2, dtype=np.float64)
        if ABCD.shape[1] != ABCD.shape[0] + nu:
            raise ValueError('The ABCD argument does not have proper dimensions.')
        form = 1
    else:
        zeros, poles, k = _get_zpk(arg2)
        form = 2
    #raise TypeError('%s: Unknown transfer function %s' % (__name__, str(arg2)))
        
    # need to set order and form now.
    order = carray(zeros).shape[0] if form == 2 else ABCD.shape[0] - nq
    
    if not isinstance(x0, collections.Iterable):
        x0 = x0*np.ones((order, 1), dtype=np.float64)
    else:
        x0 = np.array(x0).reshape((-1, 1))
    
    if form == 1:
        A = ABCD[:order, :order]
        B = ABCD[:order, order:order+nu+nq]
        C = ABCD[order:order+nq, :order]
        D1 = ABCD[order:order+nq, order:order+nu]
    else:
        A, B2, C, D2 = zpk2ss(poles, zeros, -1)    # A realization of 1/H
        # Transform the realization so that C = [1 0 0 ...]
        C, D2 = np.real_if_close(C), np.real_if_close(D2)
        Sinv = orth(np.hstack((np.transpose(C), np.eye(order)))) / norm(C)
        S = inv(Sinv)
        C = np.dot(C, Sinv)
        if C[0, 0] < 0:
            S = -S
            Sinv = -Sinv
        A = np.dot(np.dot(S, A), Sinv) 
        B2 = np.dot(S, B2) 
        C = np.hstack((np.ones((1, 1)), np.zeros((1, order-1)))) # C=C*Sinv; 
        D2 = np.zeros((0,))
        # !!!! Assume stf=1
        B1 = -B2
        D1 = 1
        B = np.hstack((B1, B2))

    N = u.shape[1]
    v = np.empty((nq, N), dtype=np.float64)
    y = np.empty((nq, N), dtype=np.float64)     # to store the quantizer input
    xn = np.empty((order, N), dtype=np.float64) # to store the state information
    xmax = np.abs(x0) # to keep track of the state maxima

    for i in range(N):
        # y0 needs to be cast to real because ds_quantize needs real
        # inputs. If quantization were defined for complex numbers,
        # this cast could be removed
        y0 = np.real(np.dot(C, x0) + np.dot(D1, u[:, i]))
        y[:, i] = y0
        v[:, i] = ds_quantize(y0, nlev)
        x0 = np.dot(A, x0) + np.dot(B, np.vstack((u[:, i], v[:, i])))
        xn[:, i] = np.real_if_close(x0.T)
        xmax = np.max(np.hstack((np.abs(x0), xmax)), axis=1, keepdims=True)

    return v.squeeze(), xn.squeeze(), xmax, y.squeeze()

def ds_quantize(y, n):
    """v = ds_quantize(y,n)
    Quantize y to:
     
    * an odd integer in [-n+1, n-1], if n is even, or
    * an even integer in [-n, n], if n is odd.

    This definition gives the same step height for both mid-rise
    and mid-tread quantizers.
    """
    v = np.zeros(y.shape)
    for qi in range(n.shape[0]): 
        if n[qi] % 2 == 0: # mid-rise quantizer
            v[qi, 0] = 2*np.floor(0.5*y[qi, 0]) + 1
        else: # mid-tread quantizer
            v[qi, 0] = 2*np.floor(0.5*(y[qi, 0] + 1))
        L = n[qi] - 1
        v[qi, 0] = np.sign(v[qi, 0])*np.min((np.abs(v[qi, 0]), L))
    return v



# -*- coding: utf-8 -*-
# test_ds_hann.py
# This module provides the tests for the ds_hann() function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the ds_hann() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestDs_Hann(unittest.TestCase):
    """Test class for ds_hann()"""
    def setUp(self):
        self.res = np.array([0.        ,  0.02148628,  0.06768441,  0.09549150,
                             0.06533781, -0.03015369, -0.1545085 , -0.24133259,
                             -0.22851372, -0.0954915 ])

    def test_ds_hann(self):
        """Test function for ds_hann()"""
        self.assertTrue(np.allclose(self.res, np.hanning(10) - ds.ds_hann(10), atol=1e-8, rtol=1e-5))
        




# -*- coding: utf-8 -*-
# test_lollipop.py
# This module provides the tests for the lollipop function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the lollipop() function.
"""

import unittest
import numpy as np
import pylab as plt

from warnings import catch_warnings

from deltasigma import lollipop

class TestLollipop(unittest.TestCase):
    """Test class for lollipop()"""

    def setUp(self):
        pass

    def test_lollipop(self):
        """Test function for lollipop()"""
        t = np.arange(1, 20)*1e-3
        f = 20.
        a = np.sin(2*np.pi*f*t)
        plt.figure()
        with catch_warnings(record=True) as w:
            lollipop(t, a, color=None, lw=1.5, ybot=0.1)
            self.assertTrue(len(w) > 0)




# -*- coding: utf-8 -*-
# test_PlotExampleSpectrum.py
# This module provides the tests for the PlotExampleSpectrum function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the PlotExampleSpectrum() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestPlotExampleSpectrum(unittest.TestCase):
    """Test class for PlotExampleSpectrum()"""

    def setUp(self):
        pass

    def test_PlotExampleSpectrum(self):
        """Test function for PlotExampleSpectrum()"""
        order = 3
        osr = 32
        f0 = 0.
        Hinf = 1.5
        ntf = ds.synthesizeNTF(order, osr, 0, Hinf, f0)
        ret = ds.PlotExampleSpectrum(ntf, M=1, osr=osr, f0=f0)
        self.assertIsNone(ret)




# -*- coding: utf-8 -*-
# _rms.py
# This module provides the rms function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the rms() function, which calculates the Root Mean 
Square (RMS) of a vector.
"""

import numpy as np
import numpy.linalg as la


def rms(x, no_dc=False):
    """Calculate the RMS value of ``x``.

    The Root Mean Square value of an array :math:`x` of length :math:`n` is defined as:

    .. math::

        x_{RMS} = \\sqrt{\\frac{1}{n}(x_1^2 + x_2^2 + ...+x_n^2)}

    **Parameters:**

    x : (N,) ndarray
        The input vector

    no_dc : boolean, optional
        If set to ``True``, the DC value gets subtracted from ``x`` first and the RMS is computed on the result.

    **Returns:**

    xrms : scalar
        as defined above

    """
    if no_dc:
        x = x - np.mean(x)
    return la.norm(x)/np.sqrt(max(x.shape))



# -*- coding: utf-8 -*-
# _evalF0.py
# Module providing the evalF0 function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the evalF0() function
"""

from __future__ import division

from ._evalF1 import evalF1


def evalF0(f1, z, phi):
    """Calculate the values of the F0 (prototype) filter 
    of a Saramaki HBF at the given points.
    """
    return evalF1(f1, 0.5*(z + 1./z), phi)



# -*- coding: utf-8 -*-
# _padl.py
# This module provides the padl function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the padl() function, which pads a matrix on the 
left.
"""

import numpy as np


def padl(x, n, val=0.):
    """Pad a matrix ``x`` on the left to length ``n`` with value ``val``.

    **Parameters:**

    x : ndarray
        The matrix to be padded.

    n : int
        The number of colums of the matrix after padding.

    val : scalar, optional
        The value to be used used for padding.

    .. note:: A 1-d array, for example ``a.shape == (N,)`` is reshaped to be
              a 1 row array: ``a.reshape((1, N))``

    The empty matrix is assumed to be have 1 empty row.

    **Returns:**

    xp : 2-d ndarray
        The padded matrix.
    """
    if len(x.shape) == 1:
        xp = x.reshape((1, x.shape[0]))
    else:
        xp = x
    y = np.concatenate(( 
                        val*np.ones((xp.shape[0], n - xp.shape[1])),
                        xp
                       ), axis=1)
    return y



# -*- coding: utf-8 -*-
# _undbm.py
# This module provides the undbm function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the undbm() function.
"""

import numpy as np

from ._utils import carray, restore_input_form, save_input_form


def undbm(p, z=50):
    """Calculate the RMS voltage equivalent of a power ``p`` expressed in dBm.

    .. math::

        V_{\\mathrm{RMS}} = \\sqrt{z\\ 10^{p/10 - 3}}

    **Parameters:**

    p : scalar or sequence
        The power to be converted.

    z : scalar, optional
        The normalization resistance value, defaults to 50 ohm.

    **Returns:**

    Vrms : scalar or sequence
           The RMS voltage corresponding to p

    .. seealso:: :func:`undbp`, :func:`undbv`, :func:`dbm`, :func:`db`

    """
    iform = save_input_form(p)
    p = carray(p)
    up = np.sqrt(z*10.**(p/10.-3))
    return restore_input_form(up, iform)



# -*- coding: utf-8 -*-
# test_mapRtoQ.py
# This module provides the tests for the mapRtoQ function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the mapRtoQ() function.
"""

from __future__ import division

import unittest
import numpy as np
import deltasigma as ds

class TestMapRtoQ(unittest.TestCase):
    """Test class for mapRtoQ()"""

    def setUp(self):
        self.test_matrix = np.arange(1, 25).reshape((4, 6)).T
        self.dq = np.array([[4.5 - 2.5j, 16.5 - 2.5j],
                            [6.5 - 2.5j, 18.5 - 2.5j],
                            [8.5 - 2.5j, 20.5 - 2.5j]])
        self.dp = np.array([[-3.5 + 4.5j, -3.5 + 16.5j],
                            [-3.5 + 6.5j, -3.5 + 18.5j],
                            [-3.5 + 8.5j, -3.5 + 20.5j]])

    def test_mapRtoQ(self):
        """Test function for mapRtoQ()"""
        resq, resp = ds.mapRtoQ(self.test_matrix)
        self.assertTrue(np.allclose(resq, self.dq, atol=1e-8, rtol=1e-5))
        self.assertTrue(np.allclose(resp, self.dp, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# _calculateSNR.py
# Module providing the calculateSNR function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the calculateSNR() function
"""

from __future__ import division, print_function

import numpy as np
from numpy.linalg import norm

from ._dbv import dbv


def calculateSNR(hwfft, f, nsig=1):
    """Estimate the SNR from the FFT.

    Estimate the Signal-to-Noise Ratio (SNR), given the in-band bins of
    a Hann-windowed FFT and the location ``f0`` of the input signal (f>0).
    For ``nsig = 1``, the input tone is contained in ``hwfft(f:f+2)``,
    this range is appropriate for a Hann-windowed FFT.

    Each increment in ``nsig`` adds a bin to either side.

    The SNR is expressed in dB.

    **Parameters:**

    hwfft : sequence
            the FFT

    f : integer
        Location of the input signal. Normalized.

    .. note:: f = 0 corresponds to DC, as Python indexing starts from 0.

    nsig : integer, optional
           Extra bins added to either side. Defaults to 1.

    **Returns:**

    SNR : scalar
          The computed SNR value in dB.

    """
    hwfft = hwfft.squeeze()
    signalBins = np.arange(f - nsig + 1, f + nsig + 2, dtype='int64')
    signalBins = signalBins[signalBins > 0]
    signalBins = signalBins[signalBins <= max(hwfft.shape)]
    s = norm(hwfft[signalBins - 1]) # *4/(N*sqrt(3)) for true rms value;
    noiseBins = np.arange(1, max(hwfft.shape) + 1, dtype='int64')
    noiseBins = np.delete(noiseBins, noiseBins[signalBins - 1] - 1)
    n = norm(hwfft[noiseBins - 1])
    if n == 0:
        snr = np.Inf
    else:
        snr = dbv(s/n)
    return snr



# -*- coding: utf-8 -*-
# _logsmooth.py
# Module providing the logsmooth function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the logsmooth() function
"""

from __future__ import division

import numpy as np
from scipy.linalg import norm

from ._dbp import dbp


def logsmooth(X, inBin, nbin=8, n=3):
    """Smooth the fft, and convert it to dB.

    **Parameters:**

    X : (N,) ndarray
        The FFT data.

    inBin : int
        The bin index of the input sine wave (if any).

    nbin : int, optional
        The number of bins on which the averaging will be performed,
        used *before* 3*inBin

    n : int, optional
        Around the location of the input signal and its harmonics (up to the
        third harmonic), don't average for n bins.

    The logsmooth algorithm uses nbin bins from 0 to 3*inBin,
    thereafter the bin sizes is increased by a factor of 1.1,
    staying less than 2^10.

    For the :math:`n` sets of bins:
    :math:`inBin + i, 2*inBin + i ... n*inBin+i`, where :math:`i \\in [0,2]`
    don't do averaging. This way, the noise BW
    and the scaling of the tone and its harmonics are unchanged.

    .. note::

        Unfortunately, harmonics above the nth appear smaller than they
        really are because their energy is averaged over many bins.

    **Returns:**

    f, p : tuple of 1d- ndarrays
        The bins and smoothed FFT, expressed in dB.

    .. seealso::

         * :func:`plotSpectrum`, convenience function to first call
           :func:`logsmooth` and then plot on a logarithmic x-axis its return
           value.

         * :func:`circ_smooth`, smoothing algorithm suitable for linear
           x-axis plotting.

    .. plot::

        import matplotlib.pyplot as plt
        import numpy as np
        from deltasigma import dbv, ds_hann, figureMagic, logsmooth
        T = 2 #s
        Fs = 231e3 #Hz
        N = int(np.round(T*Fs, 0)) # FFT points
        freq = .1e3
        t = np.arange(N)/Fs
        u0 = np.sin(2*np.pi*t*freq)
        u0 = u0 + .01*u0**2+.001*u0**3+.005*u0**4
        U = np.fft.fft(u0 * ds_hann(N))/(N/4)
        f = np.linspace(0, Fs, N + 1)
        f = f[:N/2 + 1]
        plt.subplot(211)
        plt.semilogx(f, dbv(U[:N/2 + 1]))
        plt.hold(True)
        inBin = np.round(freq/Fs*N)
        fS, US = logsmooth(U, inBin)
        plt.semilogx(fS*Fs, US, 'r', linewidth=2.5)
        plt.xlim([f[0]*Fs, Fs/2])
        plt.ylabel('U(f) [dB]')
        figureMagic(xRange=[100, 1e4], yRange=[-400, 0], name='Spectrum')
        plt.subplot(212)
        plt.loglog(fS[1:]*Fs, np.diff(fS*Fs))
        plt.xlabel('f [Hz]')
        plt.ylabel('Averaging interval [Hz]')
        figureMagic(xRange=[100, 1e4])
        plt.show()

    """
    # preliminary sanitization of the input
    if not np.prod(X.shape) == max(X.shape):
        raise ValueError('Expected a (N,) or (N, 1)-shaped array.')
    if len(X.shape) > 1:
        X = np.squeeze(X)
    inBin = int(inBin)

    N = X.shape[0]
    N2 = int(np.floor(N/2))
    f1 = int(inBin % nbin)
    startbin = np.concatenate((np.arange(f1, inBin, nbin),
                               np.arange(inBin, inBin + 3)
                              ))
    i = 1 # my fix
    while i < n: # n can be big and xrange is not in Python3
        startbin = np.concatenate((startbin,
                                   np.arange(startbin[-1] + 1,
                                             (inBin + 1)*(i + 1) - 1, nbin),
                                   (i + 1)*(inBin + 1) - 1 + np.arange(0, 3)
                                  ))
        i = i + 1
    # the following is my fix - REP?
    startbin = np.concatenate((startbin, np.array((startbin[-1] + 1,))))
    m = startbin[-1] + nbin
    while m < N2 - 1:
        startbin = np.concatenate((startbin, np.array((m,))))
        nbin = np.min((nbin*1.1, 2**10))
        m = int(np.round(m + nbin, 0))

    stopbin = np.concatenate((startbin[1:] - 1, np.array((N2 - 1,))))
    f = ((startbin + stopbin)/2)/N
    p = np.zeros(f.shape)
    for i in range(f.shape[0]):
        p[i] = dbp(norm(X[startbin[i]:stopbin[i] + 1])**2/
                   (stopbin[i] - startbin[i] + 1))
    return f, p



# -*- coding: utf-8 -*-
# _evalF1.py
# Module providing the evalF1 function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the evalF1() function
"""
from __future__ import division

import numpy as np


def evalF1(f1, z, phi=None):
    """Calculate the values of the F1 filter 
    (tranformed prototype filter) of a Saramaki HBF at the given points.
    """
    if phi is not None:
        z = z/phi
    f1 = np.asarray(f1).squeeze()
    f1 = np.atleast_1d(f1)

    F1 = 0.5
    for i in range(f1.shape[0]):
        F1 = F1 + f1[i]*z**(2*i+1)

    return F1



# -*- coding: utf-8 -*-
# _predictSNR.py
# Module providing predictSNR
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the predictSNR function.
"""


from __future__ import division

import numpy as np
from scipy.interpolate import interp1d
from scipy.signal import dimpulse, freqz
from scipy.special import erfinv

from ._dbp import dbp
from ._utils import _get_num_den


def predictSNR(ntf, OSR=64, amp=None, f0=0.):
    """Predict the SNR curve of a binary delta-sigma modulator.

    The prediction is performed using the describing function method of Ardalan
    and Paulos [2]_ .

    **Parameters:**
    
    ntf : lti object, or zpk or (num, den) or (A,B,C,D) tuples
        The noise transfer function specifying the modulator.
    
    OSR : scalar, optional
        The oversampling ratio, defaults to 64.
    
    amp : ndarray-like, optional
        The magnitudes to be used for the input signal. They are expressed in 
        dB, where 0 dB means a full-scale (peak value = 1) sine wave. 
        Defaults to [-120 -110 ... -20 -15 -10 -9 -8 ... 0].
    
    f0 : scalar, optional 
        The normalized input signal frequency. Defaults to 0.
    
    **Notes:**
    
    The band of interest is defined by the oversampling ratio (``OSR``) and
    the center frequency (``f0``).

    The algorithm assumes that the ``amp`` vector is sorted in increasing order;
    once instability is detected, the remaining SNR values are set to ``-Inf``.
    
    Future versions may accommodate STFs.

    **Returns:**

    snr : ndarray
        A vector of SNR values, in dB.

    amp : ndarray
        A vector of amplitudes, in dB.

    k0 : ndarray
        The quantizer signal gain.

    k1: ndarray
        The quantizer noise gain.

    sigma_e2 : scalar
        The power of the quantizer noise (not in dB).
    

    .. rubric:: Implementation details:

    The describing function method of A&P treats the quantizer processes
    signal and noise components separately. The quantizer is modelled as two
    (not necessarily equal) linear gains, :math:`k_0` (``k0`` in the code) 
    and :math:`k_1` (``k1``), and an additive white gaussian noise source of
    power :math:`\\sigma_e^2` (``sigma_e2``), as shown in the figure below. 

    :math:`k_0`, :math:`k_1` and :math:`\\sigma_e^2` are calculated as
    functions of the input.

    .. image:: ../doc/_static/predictSNR.png
        :align: center
        :alt: modulator model for predictSNR


    The modulator's loop filter is assumed to have nearly infinite gain at
    the test frequency.

    .. rubric:: Example:

    See :func:`simulateSNR` for an example use of this function.

    .. rubric:: References

    .. [2] Ardalan, S.H.; Paulos, J.J., "An analysis of nonlinear behavior in
           delta - sigma modulators," Circuits and Systems, IEEE Transactions
           on, vol.34, no.6, pp.593,603, Jun 1987
    
    """

    # extract num, den
    if (hasattr(ntf, 'inputs') and not ntf.inputs == 1) or \
       (hasattr(ntf, 'outputs') and not ntf.outputs == 1):
        raise TypeError("The supplied TF isn't a SISO transfer function.")

    num, den = _get_num_den(ntf)
    Nb = 100
    if f0 == 0:
        band_of_interest = np.linspace(0, np.pi/OSR, Nb)
    else:
        band_of_interest = np.linspace(2*np.pi*(f0 - 0.25/OSR), 2*np.pi*(f0 + 0.25/OSR), Nb)
        XTAB = np.linspace(-2, 0, 21)
        YTAB = np.array([
            [0.46575960516930,   0.67366999387741],
            [0.47904652357101,   0.68426650762558],
            [0.49316295981407,   0.69527947902679],
            [0.50817364454269,   0.70673173666000],
            [0.52414894104004,   0.71864765882492],
            [0.54116523265839,   0.73105299472809],
            [0.55930554866791,   0.74397552013397],
            [0.57866013050079,   0.75744456052780],
            [0.59932720661163,   0.77149158716202],
            [0.62141352891922,   0.78615015745163],
            [0.64503526687622,   0.80145609378815],
            [0.67031890153885,   0.81744754314423],
            [0.69740217924118,   0.83416539430618],
            [0.72643494606018,   0.85165339708328],
            [0.75758063793182,   0.86995816230774],
            [0.79101717472076,   0.88912981748581],
            [0.82693856954575,   0.90922164916992],
            [0.86555624008179,   0.93029111623764],
            [0.90710091590881,   0.95239937305450],
            [0.95182400941849,   0.97561222314835],
            [1.00000000000000,   1.00000000000000]])

    if amp is None:
        amp = np.concatenate((np.arange(- 120, -20 + 1, 10),
                              np.array((-15,)),
                              np.arange(-10, 1)
                            ))
    num = np.real_if_close(num)
    den = np.real_if_close(den)
    num1 = num - den
    N = max(amp.shape)
    snr = np.zeros((1, N)) - np.Inf
    k0 = np.zeros((1, N))
    k1 = np.zeros((1, N))
    sigma_e2 = np.zeros((1, N))
    u = 10.0**(amp/20)
    Nimp = 100
    unstable = False
    for n in range(N):
        # Calculate sigma_e2
        if f0 == 0:
            erfinvu = erfinv(u[n])
            sigma_e2[0, n] = 1 - u[n]**2 - 2/np.pi * np.exp(-2*erfinvu**2)
        else:
            # % Sinusoidal input.
            # Solve sqrt(pi)*u/2 = rho * hypergeo(0.5,2,-rho^2);
            # Formulate as solve f(rho) = 0, f = rho*M(0.5,2,-rho^2)-K
            # and use the secant method.
            K = 0.5*np.sqrt(np.pi)*u[n]
            if n == 0:
                # Initial guess; otherwise use previous value.
                rho = u[n]**2
                fprime = 1
            drho = 1
            f_prev = None
            for itn in range(0, 20):
                m0 = interp1d(XTAB, YTAB[:, 1], kind='cubic')(-rho**2)
                f = rho*m0 - K
                if itn > 0:
                    fprime = max((f - f_prev)/drho, 0.5) #Secant approx.
                if abs(f) < 1e-08:
                    break #!Converged
                drho = -f/fprime
                if abs(drho) > 0.2:
                    drho = np.sign(drho) * 0.2
                if abs(drho) < 1e-06:
                    break #!Converged
                rho = rho + drho
                f_prev = f
            m1 = interp1d(XTAB, YTAB[:, 0], kind='cubic')(-rho**2)
            sigma_e2[0, n] = 1 - u[n]**2/2 - 2/np.pi*m1**2
        # Iterate to solve for k1 and sigma_1.
        # Using one of MATLAB's nonlinear equation solvers would be more efficient,
        # but this function code would then require the optimization toolbox.
        # !Future work: put in 2-D BFGS code.
        if n > 0:
            #  Use the previous value of k1 as the initial guess.
            k1[0, n] = k1[0, n - 1]
        else:
            k1[0, n] = 1.2
        k1_prev = 0
        itn = 0
        if f0 == 0:
            k1sigma1 = np.sqrt(2/np.pi) * np.exp(-erfinvu**2)
        else:
            k1sigma1 = np.sqrt(2/np.pi)*m1
        while abs(k1[0, n] - k1_prev) > 1e-06*(1 + k1[0, n]) and itn < 100:
            #  Create the function: H_hat = L1/(1-k1*L1)=(H-1)/(H*(1-k1)+k1).
            den1 = (1 - k1[0, n])*num + den*k1[0, n]
            #  Calculate pGain, the square of the 2-norm of H_hat.
            pGain, Nimp = powerGain(num1, den1, Nimp)
            if np.isinf(pGain):
                unstable = True
                break
            sigma_1 = np.sqrt(pGain * sigma_e2[0, n])
            k1_prev = k1[0, n]
            k1[0, n] = k1sigma1/sigma_1
            itn = itn + 1

        if unstable:
            break
        if f0 == 0:
            y0 = np.sqrt(2)*erfinvu*sigma_1
            k0[0, n] = u[n]/y0
        else:
            k0[0, n] = np.sqrt(2/np.pi)*m0/sigma_1
        _, h = freqz(num, (1 - k1[0, n])*num + k1[0, n]*den, band_of_interest)
        # For both DC and sine wave inputs, use u^2/2 as the signal 
        # power since true DC measurements are usually impossible.
        snr[0, n] = dbp(0.5*u[n]**2/(np.sum(h**2)/(OSR*Nb)*sigma_e2[0, n]))
    return snr.squeeze(), amp.squeeze(), k0.squeeze(), k1.squeeze(), sigma_e2.squeeze()

def powerGain(num, den, Nimp=100):
    """Calculate the power gain of a TF given in coefficient form.

    Nimp is the recommended number of impulse response samples for use
    in future calls and Nimp0 is the suggested number (100) to use.
    """
    unstable = False
    _, (imp, ) = dimpulse((num, den, 1), t=np.linspace(0, Nimp, Nimp))
    if np.sum(abs(imp[Nimp - 11:Nimp])) < 1e-08 and Nimp > 50:
        Nimp = np.round(Nimp/1.3)
    else:
        while np.sum(abs(imp[Nimp - 11:Nimp])) > 1e-06:
            Nimp = Nimp*2
            _, (imp, ) = dimpulse((num, den, 1), t=np.linspace(0, Nimp, Nimp))
            if np.sum(abs(imp[Nimp - 11:Nimp])) >= 50 or Nimp >= 10000.0:
                unstable = True
                break

    if not unstable:
        pGain = np.sum(imp**2)
    else:
        pGain = np.Inf

    return pGain, Nimp



# -*- coding: utf-8 -*-
# _sinc_decimate.py
# This module provides the sinc_decimate function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the sinc_decimate() function, which decimates a vector 
by a sinc filter of specified order and length.
"""

from __future__ import division

import numpy as np


def sinc_decimate(x, m, r):
    """Decimate ``x`` by an ``m``-th order sinc filter of length ``r``.
    """
    x = x[:]
    if len(x) < r:
        raise ValueError('Filter length longer than input data ( %d > %d )' %
                         (r, len(x)))
    for _ in range(m):
        x = np.cumsum(x)
        x = np.concatenate((x[:r], x[r:] - x[:-r]), axis=0)/r
    return x[r-1::r]



# -*- coding: utf-8 -*-
# _ds_synNTFobj1.py
# Module providing the ds_synNTFobj1 function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the ds_synNTFobj1() function
"""

import numpy as np

from ._db import db
from ._ds_f1f2 import ds_f1f2
from ._padb import padb
from ._padt import padt
from ._rmsGain import rmsGain
from ._utils import carray


def ds_synNTFobj1(x, p, osr, f0):
    """Objective function for :func:`synthesizeNTF`

    This function is not meant to be used directly but it is provided for compliance with the 
    MATLAB DS Toolbox.

    """
    p = carray(p)
    z = np.exp(2j*np.pi*(f0 + 0.5/osr*x))
    z = carray(z)
    if f0 > 0:
        z = padt(z, p.shape[0]/2., np.exp(2j*np.pi*f0))

    z = np.hstack((z, np.conj(z))) 
    z = z[:]
    if f0 == 0:
        z = padb(z, p.shape[0], 1)

    f1, f2 = ds_f1f2(osr, f0)
    ntf = (z, p, 1)
    y = db(rmsGain(ntf, f1, f2))
    return y



# -*- coding: utf-8 -*-
# _axisLabels.py
# Module providing the axisLabel function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the axisLabel() function
"""

import collections

import numpy as np


def axisLabels(ran, incr):
    """Utility function to quickly generate the alphanum. axis labels.

    **Parameters:**

    ran : sequence
        Sequence containing the axis points (floats)

    incr : int, or 2-elements sequence
        This parameter may be:

    * an int, the function returns an array of strings corresponding to:
      each element of ``range[0]:range[-1]:incr`` formatted as ``'%g'``.

    * a list, the function returns an array of strings corresponding to:
      each element of ``incr[1]:range[-1]:incr[0]`` formatted as ``'%g'``.

    .. note:: All elements in ``ran`` less than 1e-6 are rounded down to 0.

    **Returns:**

    labels : list of strings

    **Raises:**

    ValueError: "Unrecognised incr."
    """
    ran = np.asarray(ran)
    ran[np.abs(ran) < 1e-6] = 0
    s = []
    if not isinstance(incr, collections.Iterable):
        incr = int(incr)
        first = 0
    elif len(incr) == 2:
        first = incr[1]
        incr = incr[0]
    else:
        raise ValueError("Unrecognised incr: " + str(incr))
    for i in range(first, len(ran), incr):
        s += ['%g' % ran[i]]
    return s



# -*- coding: utf-8 -*-
# test_calculateSNR.py
# This module provides the tests for the calculateSNR() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds
from numpy.fft import fft
from deltasigma._ds_hann import ds_hann


class TestCalculateSNR(unittest.TestCase):
    """Test function for calculateSNR()"""
    def setUp(self):
        N = 2**12
        t = np.arange(N)
        (f1, f2) = (1.0/8, 1.0/302)
        A = np.cos(2*np.pi*f1*t)
        B = 0.01*np.cos(2*np.pi*f2*t)
        y = A + B
        window = ds_hann(N)
        self.hwfft = fft(window*y)
        self.N = N
        self.f1 = f1

    def test_snr_is_40(self):
        """ Test that a particular SNR is within roundings errors of
        40 (dB?) """
        N = self.N
        snr = ds.calculateSNR(self.hwfft[:N/2], int(N*self.f1))
        # Consider replacing with assertAlmostEqual
        self.assertTrue(np.allclose(snr, 40, atol=1e-8, rtol=1e-8))

    def test_snr_is_inf(self):
        """ Test that a paricular SNR is infinite. """
        N = self.N
        hwfft = np.zeros((N/2, ))
        hwfft[512] = 1.0  # specially crafted to have Inf snr
        snr = ds.calculateSNR(hwfft[:N/2], 512)
        self.assertEqual(snr, np.Inf)



# -*- coding: utf-8 -*-
# _bplogsmooth.py
# Module providing the bplogsmooth function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the bplogsmooth() function, which smooths an fft and 
converts it to dB.
"""

from __future__ import division

import numpy as np
from numpy.linalg import norm

from ._dbp import dbp
from ._utils import carray, mround


def bplogsmooth(X, tbin, f0):
    """Smooth the FFT and convert it to dB.

    Use 8 bins from the bin corresponding to ``f0`` to ``tbin`` and again as far.
    Thereafter increase bin sizes by a factor of 1.1, staying less than 2^10.
    For tbin, group the bins together.

    Use this for nice double-sided log-log plots.

    .. note:: ``tbin`` is assumed to be in the upper sideband!

    .. seealso:: :func:`logsmooth`

    """
    X = carray(X).squeeze()
    if len(X.shape) > 1:
        raise ValueError("The X vector is not unidimensional: " + str(X.shape))

    N = X.shape[0]
    N2 = int(np.floor(N/2))
    tbin = int(tbin)
    n = 8

    bin0 = int(mround(f0*N))
    assert tbin > bin0 # we said upper sideband!
    bin1 = ((tbin - bin0) % n) + bin0
    bind = bin1 - bin0
    usb1 = np.concatenate((np.arange(bin1, tbin+1, n), 
                           np.arange(tbin+3, tbin+bind+1, 8)
                         ))
    m = usb1[-1] + n
    while m + n/2. < N/2.:
        usb1 = np.concatenate((usb1, np.array((m,))))
        n = mround(min(n*1.1, 2**10))
        m = m + int(n)
    usb2 = np.concatenate((usb1[1:]-1, np.array((N2,))))

    n = 8
    lsb2 = np.arange(bin1, bin1 - 2*bind + 1, -n) - 1
    m = lsb2[-1] - n
    while m - n/2. > 1:
        lsb2 = np.concatenate((lsb2, np.array((m,))))
        n = mround(min(n*1.1, 2**10))
        m = m - int(n)
    lsb1 = np.concatenate((lsb2[1:] + 1, np.ones((1,))))

    startbin = np.concatenate((lsb1[::-1], usb1)) - 1
    stopbin = np.concatenate((lsb2[::-1], usb2)) - 1

    f = ((startbin + stopbin)/2.)/N - f0
    p = np.zeros(f.shape)
    for i in range(f.shape[0]):
        p[i] = dbp(
                   norm(X[startbin[i]:stopbin[i] + 1]**2. /
                        (stopbin[i] - startbin[i] + 1.),
                       ord=1)
                  )
    return f, p



# -*- coding: utf-8 -*-
# test_db.py
# This module provides the tests for the db function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the db() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from nose.tools import raises

class TestDB(unittest.TestCase):
    """Test class for db()"""

    def setUp(self):
        # arrays
        self.tv1 = np.array([2])
        self.r1 = np.array([3.01029996])
        # scalars
        self.tv2 = 2
        self.r2 = 3.01029996
        # tuples
        self.tv3 = 2, 2
        self.r3 = 3.01029996, 3.01029996
        # db voltage - undbv
        self.tv4 = np.array([3.0])
        pass

    def test_db_1(self):
        """Test function for db() 1/6"""
        res = ds.db(self.tv1, 'power')
        self.assertTrue(np.allclose(self.r1, res, atol=1e-8, rtol=1e-5))

    def test_db_2(self):
        """Test function for db() 2/6"""
        res = ds.db(self.tv2, 'power')
        self.assertTrue(np.allclose(self.r2, res, atol=1e-8, rtol=1e-5))

    def test_db_3(self):
        """Test function for db() 3/6"""
        res = ds.db(self.tv3, 'power')
        self.assertTrue(np.allclose(self.r3, res, atol=1e-8, rtol=1e-5))

    def test_db_4(self):
        """Test function for db() 4/6"""
        res = ds.undbv(ds.db(self.tv4, 'voltage'))
        self.assertTrue(np.allclose(self.tv4, res, atol=1e-8, rtol=1e-5))

    @raises(ValueError)
    def test_db_5(self):
        """Test function for db() 5/6"""
        res = ds.db([1], 'wrong')

    def test_db_6(self):
        """Test function for db() 6/6"""
        res = ds.db(self.tv1, 'power', R=100)
        # the R value should be ignored (warning only)
        self.assertTrue(np.allclose(self.r1, res, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# _PlotExampleSpectrum.py
# Miscellaneous functions and stdlib wrappers for MATLAB functions
# that do not find a direct replacement in numpy/scipy.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"),  upon which it is heavily based.
# The delta sigma toolbox is (c) 2009,  Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the PlotExampleSpectrum function.
"""

from __future__ import division

import numpy as np
import pylab as plt
from numpy.fft import fft, fftshift

from ._calculateSNR import calculateSNR
from ._circ_smooth import circ_smooth
from ._dbp import dbp
from ._dbv import dbv
from ._ds_f1f2 import ds_f1f2
from ._ds_hann import ds_hann
from ._evalTF import evalTF
from ._figureMagic import figureMagic
from ._simulateDSM import simulateDSM
from ._undbv import undbv


def PlotExampleSpectrum(ntf, M=1, osr=64, f0=0, quadrature=False):
    """Plot a spectrum suitable to exemplify the NTF performance.

    **Parameters:**

    ntf : scipy 'lti' object, tuple or ndarray
           The first argument may be one of the various supported
           representations for a (SISO) transfer function or an
           ABCD matrix. See :func:`evalTF` for a more detailed
           discussion.

    f0 : float, optional
         The center frequency. Normalized. Defaults to 0.

    M : int, optional
        M is defined as:

        .. math::

            M = n_{lev} - 1

        The number of quantizer levels (:math:`n_{lev}`) defaults to 2 and
        ``M`` defaults to 1.

    quadrature : boolean, optional
                 Whether the delta sigma modulator is a quadrature
                 modulator or not. Defaults to ``False``.

    .. note::

        Quadrature modulators require the following currently unimplemented
        functions: :func:`simulateQDSM` (expected v. 0.2).
        Setting ``quadrature`` to ``True`` results in a
        ``NotImplementedError`` being raised.

    .. plot::

        import pylab as plt
        from deltasigma import synthesizeNTF, PlotExampleSpectrum
        order = 3
        osr = 32
        f0 = 0.
        Hinf = 1.5
        ntf = synthesizeNTF(order, osr, 0, Hinf, f0)
        plt.figure(figsize=(12, 5))
        PlotExampleSpectrum(ntf, M=1, osr=osr, f0=f0)

    """
    f1, f2 = ds_f1f2(osr, f0, quadrature)
    delta = 2
    Amp = undbv(-3) # Test tone amplitude, relative to full-scale.
    # f below is the test tone frequency offset from f0, relative to bw.
    # (It will be adjusted to be an fft bin)
    f = 0.3
    N = 2**12
    f1_bin = np.round(f1*N)
    f2_bin = np.round(f2*N)
    fin = round(((1 - f)/2*f1 + (f + 1)/2*f2) * N)
    if not quadrature:
        t = np.arange(0, N).reshape((1, -1))
        u = Amp*M*np.cos((2*np.pi/N)*fin*t)
        v, _, xmax, y = simulateDSM(u, ntf, M+1)
    else:
        raise NotImplementedError("The required simulateQDSM function " + \
                                  "is not available yet.")
        t = np.arange(0, N).reshape((1, -1))
        u = Amp*M*np.exp((2j*np.pi/N)*fin*t)
        v, xn, xmax, y = simulateQDSM(u, ntf, M + 1)
    window = ds_hann(N)
    NBW = 1.5/N
    spec0 = fft(v * window)/(M*N/4)
    if not quadrature:
        freq = np.linspace(0, 0.5, N/2 + 1)
        plt.plot(freq, dbv(spec0[:N/2 + 1]), 'c', linewidth=1)
        plt.hold(True)
        spec_smoothed = circ_smooth(np.abs(spec0)**2., 16)
        plt.plot(freq, dbp(spec_smoothed[:N/2 + 1]), 'b', linewidth=3)
        Snn = np.abs(evalTF(ntf, np.exp(2j*np.pi*freq)))**2 * 2/12*(delta/M)**2
        plt.plot(freq, dbp(Snn*NBW), 'm', linewidth=1)
        snr = calculateSNR(spec0[f1_bin:f2_bin + 1], fin - f1_bin)
        msg = 'SQNR  =  %.1fdB\n @ A = %.1fdBFS & osr = %.0f\n' % \
              (snr, dbv(spec0[fin]), osr)
        if f0 < 0.25:
            plt.text(f0 + 1 / osr, - 15, msg, horizontalalignment='left',
                     verticalalignment='center')
        else:
            plt.text(f0 - 1 / osr, - 15, msg, horizontalalignment='right',
                     verticalalignment='center')
        plt.text(0.5, - 135, 'NBW = %.1e ' % NBW, horizontalalignment='right',
                 verticalalignment='bottom')
        figureMagic((0, 0.5), 1./16, None, (-140, 0), 10, None)
    else:
        spec0 = fftshift(spec0 / 2)
        freq = np.linspace(-0.5, 0.5, N + 1)
        freq = freq[:-1]
        plt.plot(freq, dbv(spec0), 'c', linewidth=1)
        plt.hold('on')
        spec_smoothed = circ_smooth(abs(spec0) ** 2, 16)
        plt.plot(freq, dbp(spec_smoothed), 'b', linewidth=3)
        Snn = abs(evalTF(ntf, np.exp(2j * np.pi * freq))) ** 2 * 2 / 12 * (delta / M) ** 2
        plt.plot(freq, dbp(Snn * NBW), 'm', linewidth=1)
        snr = calculateSNR(spec0[N/2 + np.arange(f1_bin, f2_bin + 1)], fin - f1_bin)
        msg = 'SQNR  =  %.1fdB\\n @ A = %.1fdBFS & osr = %.0f\\n' % \
              (snr, dbv(spec0[N/2 + fin]), osr)
        if f0 >=  0:
            plt.text(f0 - 0.05, - 15, msg, horizontalalignment='right',
                     verticalalignment='bottom')
        else:
            plt.text(f0 + 0.05, - 15, msg, horizontalalignment='left',
                     verticalalignment='bottom')
        plt.text(-0.5, -135, ' NBW = %.1e' % NBW, horizontalalignment='left',
                 verticalalignment='bottom')
        figureMagic((-0.5, 0.5), 0.125, None, (-140, 0), 10, None)
    plt.xlabel('frequency')
    return



# -*- coding: utf-8 -*-
# _ds_f1f2.py
# This module provides the ds_f1f2 function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the ds_f1f2() function.
"""

def ds_f1f2(OSR=64, f0=0., complex_flag=False):
    """[f1, f2] = ds_f1f2(OSR=64, f0=0, complex_flag=0)
    This function has no original docstring.
    """
    if complex_flag:
        f1 = f0 - 0.5/OSR
        f2 = f0 + 0.5/OSR
    else:
        if f0 > 0.25/OSR:
            f1 = f0 - 0.25/OSR
            f2 = f0 + 0.25/OSR
        else:
            f1 = 0.
            f2 = 0.5/OSR
    return f1, f2



# -*- coding: utf-8 -*-
# _dbm.py
# This module provides the dbm function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the dbm() function, used to convert an RMS voltage to dBm.
"""

from __future__ import division

import numpy as np

from ._utils import carray, restore_input_form, save_input_form


def dbm(v, R=50):
    """Calculate the dBm equivalent of an RMS voltage ``v``.

    .. math::

        P_{dBm} = 10 \\mathrm{log}_{10}(1000 \\frac{v^2}{R} )

    **Parameters:**

    v : scalar or sequence
        The voltages to be converted.

    R : scalar, optional
        The resistor value the power is calculated upon, defaults to 50 ohm.

    **Returns:**

    PdBm : scalar or sequence
           The input in dBm.

    .. seealso:: :func:`undbm`, :func:`db`, :func:`dbp`, :func:`dbv`

    """
    iform = save_input_form(v)
    v = carray(v)
    y = -np.Inf * np.ones(np.size(v))
    nonzero = (v != 0)
    y[nonzero] = 10. * np.log10(np.abs(v[nonzero] ** 2.) / R) + 30
    return restore_input_form(y, iform)



# -*- coding: utf-8 -*-
# test_evalF0.py
# This module provides the tests for the evalF0 function.
# Copyright 2014 Giuseppe Venturini and Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the evalF0() function.
"""

import unittest
import numpy as np
from math import pi

from deltasigma import evalF0

class TestEvalF0(unittest.TestCase):
    """Test class for evalF0()"""

    def setUp(self):
        self.z = np.exp(1j*2*pi*np.arange(11.0)/10.0)

    def test_evalF0_1(self):
        """Test function for evalF0() 1/2"""
        r = evalF0(1, self.z, 1)
        ref = np.array((1.5, 1.309016994374948, 0.809016994374947,
                        0.190983005625053, -0.309016994374947, -0.5,
                        -0.309016994374947, 0.190983005625052,
                        0.809016994374947, 1.309016994374947, 1.5))
        self.assertTrue(np.allclose(r, ref))

    def test_evalF0_2(self):
        """Test function for evalF0() 2/3"""
        r = evalF0(1, self.z, 0.5)
        ref = np.array((2.5, 2.118033988749895, 1.118033988749895,
                        -0.118033988749895, -1.118033988749895, -1.5,
                        -1.118033988749895, -0.118033988749895,
                        1.118033988749895, 2.118033988749895, 2.5))
        self.assertTrue(np.allclose(r, ref))

    def test_evalF0_3(self):
        """Test function for evalF0() 3/3"""
        r = evalF0(0.5, self.z, 0.5)
        ref = np.array((1.5, 1.309016994374948, 0.809016994374947,
                        0.190983005625053, -0.309016994374947, -0.5,
                        -0.309016994374947,  0.190983005625052,
                        0.809016994374947, 1.309016994374947, 1.5))
        self.assertTrue(np.allclose(r, ref))




from __future__ import division
import unittest
import numpy as np
import deltasigma as ds
import pkg_resources
import scipy as sp
import scipy.io


## Modulator realization and dynamic range scaling - # demo3
# In this ipython notebook, the following is demonstrated:
# 
#  * A 5th order delta sigma modulator is synthesized, with optimized zeros and an OSR equal to 42.
# 
#  * We then convert the synthesized NTF into `a`, `g`, `b`, `c` coefficients for the `CRFB` modulator structure.
#  
#  * The maxima for each state are evaluated.
#  
#  * The `ABCD` matrix is scaled so that the state maxima are less than the specified limit.
# 
#  * The state maxima are re-evaluated and limit compliance is checked.
# 
# **NOTE:** This is an ipython port of `dsdemo3.m`, from the
# **[MATLAB Delta Sigma Toolbox](http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox)**, 
# written by Richard Schreier.

class TestAxisLabels(unittest.TestCase):
    """dsdemo3: Delta sigma modulator synthesis"""
    def setUp(self):
        fname = pkg_resources.resource_filename(__name__, "test_data/test_dsdemo3.mat")
        self.data = sp.io.loadmat(fname)
    def test_dsdemo3(self):
        """dsdemo3 test: Delta sigma modulator synthesis"""
        order = 5
        R = 42
        opt = 1
        H = ds.synthesizeNTF(order, R, opt)

        # ## Evaluation of the coefficients for a CRFB topology
        a, g, b, c = ds.realizeNTF(H)

        # Use a single feed-in for the input
        # Lets check that stuffABCD understands that if b is scalar it means 1 feed-in.
        ABCD = ds.stuffABCD(a, g, b[0], c)
        # for passing the assertion below, we need b to have the trailing zeros
        b = np.concatenate((np.atleast_1d(b[0]), 
                            np.zeros((b.shape[0] - 1,))))
        u = np.linspace(0, 0.6, 30)
        N = 1e4
        T = np.ones((1, N))
        maxima = np.zeros((order, len(u)))
        for i in range(len(u)):
            ui = u[i]
            v, xn, xmax, _ = ds.simulateDSM(ui*T, ABCD);
            maxima[:, i] = np.squeeze(xmax)
            if any(xmax > 1e2):
                umax = ui
                u = u[:i+1]
                maxima = maxima[:, :i]
                break
        # save the maxima
        prescale_maxima = np.copy(maxima)

        # ## Scaled modulator
        # ### Calculate the scaled coefficients
        ABCDs, umax, _ = ds.scaleABCD(ABCD, N_sim=1e5)
        as_, gs, bs, cs = ds.mapABCD(ABCDs)
        # ### Calculate the state maxima
        u = np.linspace(0, umax, 30)
        N = 1e4
        T = np.ones((N,))
        maxima = np.zeros((order, len(u)))
        for i in range(len(u)):
            ui = u[i]
            v, xn, xmax, _ = ds.simulateDSM(ui*T, ABCDs)
            maxima[:, i] = xmax.squeeze()
            if any(xmax > 1e2):
                umax = ui;
                u = u[:i]
                maxima = maxima[:, :i]
                break

        self.assertTrue(np.allclose(ABCD, self.data['ABCD']))
        self.assertTrue(np.allclose(ABCDs, self.data['ABCDs'], atol=1e-2, rtol=5e-2))
        self.assertTrue(np.allclose(a, self.data['a'], atol=1e-5, rtol=1e-3))
        self.assertTrue(np.allclose(b, self.data['b'], atol=1e-5, rtol=1e-3))
        self.assertTrue(np.allclose(g, self.data['g'], atol=1e-5, rtol=1e-3))
        self.assertTrue(np.allclose(c, self.data['c'], atol=1e-5, rtol=1e-3))
        self.assertTrue(np.allclose(as_, self.data['as'], atol=1e-2, rtol=5e-3))
        self.assertTrue(np.allclose(bs, self.data['bs'], atol=5e-3, rtol=5e-3))
        self.assertTrue(np.allclose(gs, self.data['gs'], atol=5e-3, rtol=5e-3))
        self.assertTrue(np.allclose(cs, self.data['cs'], atol=3e-2, rtol=3e-2))
        self.assertTrue(np.allclose(umax, self.data['umax'], atol=5e-3, rtol=5e-3))
        self.assertTrue(np.allclose(maxima, self.data['maxima'], atol=2e-2, rtol=5e-2))




# -*- coding: utf-8 -*-
# test_bilogplot.py
# Test module for the bilogplot function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Test module for the bilogplot function
"""

from __future__ import division

import unittest
import numpy as np
import pylab as plt
import deltasigma as ds

class TestBiLogPlot(unittest.TestCase):
    """Test function for bilogplot()"""

    def setUp(self):
        pass

    def test_bilogplot(self):
        """Test function for bilogplot()"""
        f0 = 1./8
        OSR = 64
        order = 8
        N = 8192
        H = ds.synthesizeNTF(order, OSR, 1, 1.5, f0)
        fB = int(np.ceil(N/(2. * OSR)))
        ftest = int(np.round(f0*N + 1./3 * fB))
        u = 0.5*np.sin(2*np.pi*ftest/N*np.arange(N))
        v, xn, xmax, y = ds.simulateDSM(u, H)
        spec = np.fft.fft(v*ds.ds_hann(N))/(N/4)
        X = spec[:N/2 + 1]
        plt.figure()
        # graphical function: we check it doesn't fail
        ds.bilogplot(X, f0*N, ftest, (.03, .3, .3), (-140, 0, 10, 20))



# -*- coding: utf-8 -*-
# _cancelPZ.py
# Module providing the cancelPZ function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the cancelPZ() function
"""

from __future__ import division

import copy

import numpy as np
from scipy.signal import lti


def cancelPZ(arg1, tol=1e-6):
    """Cancel zeros/poles in a SISO transfer function.

    **Parameters:**

    arg1 : arguments
        The `cancelPZ` function can be called with either 1, 2, 3 or 4 arguments.

    If one argument is used, it is a scipy `lti` object.

    If more arguments are used, they should be arranged in a tuple, the 
    following gives the number of elements in the tuple and their
    interpretation:

    * 2: (numerator, denominator)
    * 3: (zeros, poles, gain)
    * 4: (A, B, C, D)

    Each argument can be an array or sequence.

    tol : float, optional
        the absolute tolerance for pole, zero cancellation. Defaults to 1e-6.

    **Returns:**

    (z, p, k) : tuple
        A tuple containing zeros, poles and gain (unchanged) after poles, zeros 
        cancellation.

    """
    if not isinstance(arg1, lti):
        arg1 = lti(*arg1)
    z = copy.copy(arg1.zeros)
    p = copy.copy(arg1.poles)
    k = arg1.gain
    for i in range(max(z.shape) - 1, 0, -1):
        d = z[i] - p
        cancel = np.nonzero(np.abs(d) < tol)[0]
        if cancel.size:
            p = np.delete(p, cancel[0])
            z = np.delete(z, i)
    return z, p, k



# -*- coding: utf-8 -*-
# test_bunquantize.py
# This module provides the tests for the bunquantize function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the bunquantize() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class TestBUnQuantize(unittest.TestCase):
    """Test class for bunquantize()"""
    def setUp(self):
        self.x = np.linspace(-10, 10, 101)

    def test_bunquantize(self):
        """Test function for bunquantize() 1/3"""
        yr = ds.bquantize(self.x)
        yv = []
        y = []
        for yi in yr:
            y += [yi.csd]
            yv += [yi.val]
        yv = np.asarray(yv)
        xres = ds.bunquantize(y)
        self.assertTrue(np.allclose(xres, yv, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# _SIunits.py
# This module provides the SIunits function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the SIunits() function, which, given an input,
returns factor (think "engineering notation") and an alphabetic
suffix.
"""

from __future__ import division

import numpy as np

from ._constants import eps


def SIunits(x):
    """Calculates the factor for representing ``x`` in engineering 
    notation.

    The factors and suffixes supported are:

    +----------+---+---------+--------+---+---------+
    |   1e-3   | m |  milli  |   1e3  | k |   kilo  |
    +----------+---+---------+--------+---+---------+
    |   1e-6   | u |  micro  |   1e6  | M |   mega  |
    +----------+---+---------+--------+---+---------+
    |   1e-9   | n |   nano  |   1e9  | G |   giga  |
    +----------+---+---------+--------+---+---------+
    |   1e-12  | p |   pico  |  1e12  | T |   tera  |
    +----------+---+---------+--------+---+---------+
    |   1e-15  | f |  femto  |  1e15  | P |   peta  |
    +----------+---+---------+--------+---+---------+
    |   1e-18  | a |   atto  |  1e18  | E |    exa  |
    +----------+---+---------+--------+---+---------+
    |   1e-21  | z |  zepto  |  1e21  | Z |   zeta  |
    +----------+---+---------+--------+---+---------+
    |   1e-24  | y |  yocto  |  1e24  | Y |  yotta  |
    +----------+---+---------+--------+---+---------+

    **Parameters:**

    x : scalar or array_like
            The number for which the engineering notation factor and suffix
        are to be calculated.

    **Returns:**

    factor : float or list of floats
             the engineering notation factor(s)

    prefix : string or list of strings
             the engineering notation unit prefix(es)

    **Example**::

        a = 3300.
        unit = 'g'
        f, p = SIunits(a)
        print "Float 'a' in engineering notation: %.3f %s%s" % (a/f, p, unit)

    Prints::

        Float 'a' in engineering notation: 3.300 kg
    """
    prefixes_n = ('', 'm', 'u', 'n', 'p', 'f', 'a', 'z', 'y')
    prefixes_p = ('', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
    scalar_input = np.isscalar(x)
    if scalar_input:
        x = [x]
    factor = np.ones(len(x))
    prefix = ['']*len(x)
    for i in range(len(x)):
        if x[i] != 0:
            p = int(np.floor(np.log10(abs(x[i]))/3 + eps))
            if p >= 0:
                p = min(p, len(prefixes_p))
                prefix[i] = prefixes_p[p]
                factor[i] = 10**(3*p)
            elif p < 0:
                p = min(-p, len(prefixes_n)-1)
                prefix[i] = prefixes_n[p]
                factor[i] = 10**(-3*p)
        else:
            factor[i], prefix[i] = 0, ''
    if scalar_input:
        factor, prefix = factor[0], prefix[0]
    return factor, prefix



# -*- coding: utf-8 -*-
# test_ds_synNTFobj1.py
# This module provides the tests for the ds_synNTFobj1() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestDSSynNTFObj1(unittest.TestCase):
    """Test functions for ds_synNTFobj1()"""

    def setUp(self):
        pass

    def test_ds_synNTFobj1_1(self):
        """Test function for ds_synNTFobj1() 1/2"""
        res = -27.167735573627283
        tv = ds.ds_synNTFobj1(.5, (.9, 2), 64, .1)
        self.assertTrue(np.allclose((res,), (tv, ), atol=1e-8, rtol=1e-5))

    def test_ds_synNTFobj1_2(self):
        """Test function for ds_synNTFobj1() 2/2"""
        res = -43.0365
        tv = ds.ds_synNTFobj1(.5, (.9, 2), 64, 0.)
        self.assertTrue(np.allclose((res,), (tv, ), atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# test_partitionABCD.py
# This module provides the tests for the partitionABCD function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the partitionABCD() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from scipy.signal import lti

class TestPartitionABCD(unittest.TestCase):
    """Test class for partitionABCD()"""

    def setUp(self):
        # data for test 1
        self.ob = lti((1, ), (1, 2, 10))
        ab = np.hstack((self.ob.A, self.ob.B))
        cd = np.hstack((self.ob.C, self.ob.D.reshape((1,1))))
        self.ABCD1 = np.vstack((ab, cd))
        # data for test2
        ABCD = [[1.000000000000000, 0., 0., 0.044408783846879, -0.044408783846879],
                [0.999036450096481, 0.997109907515262, -0.005777399147297, 0., 0.499759089304780],
                [0.499759089304780, 0.999036450096481, 0.997109907515262,  0., -0.260002096136488],
                [0, 0, 1., 0, -0.796730400347216]]
        self.ABCD2 = np.array(ABCD)
        self.at = np.array([[1., 0., 0.],
                            [0.999036450096481, 0.997109907515262, -0.005777399147297],
                            [0.499759089304780, 0.999036450096481,  0.997109907515262]
                           ])
        self.bt = np.array([[0.044408783846879, -0.044408783846879],
                            [0., 0.499759089304780],
                            [0., -0.260002096136488]
                           ])
        self.ct = np.array([0., 0, 1])
        self.dt = np.array([0., -0.796730400347216])

    def test_partitionABCD_1(self):
        """Test function for partitionABCD() 1/2"""
        a, b, c, d = ds.partitionABCD(self.ABCD1)
        self.assertTrue(np.allclose(a, self.ob.A, rtol=1e-5, atol=1e-8))
        self.assertTrue(np.allclose(b, self.ob.B, rtol=1e-5, atol=1e-8))
        self.assertTrue(np.allclose(c, self.ob.C, rtol=1e-5, atol=1e-8))
        self.assertTrue(np.allclose(d, self.ob.D, rtol=1e-5, atol=1e-8))

    def test_partitionABCD_2(self):
        """Test function for partitionABCD() 2/2"""
        ar, br, cr, dr = ds.partitionABCD(self.ABCD2, m=2)
        self.assertTrue(np.allclose(self.at, ar, rtol=1e-5, atol=1e-8))
        self.assertTrue(np.allclose(self.bt, br, rtol=1e-5, atol=1e-8))
        self.assertTrue(np.allclose(self.ct, cr, rtol=1e-5, atol=1e-8))
        self.assertTrue(np.allclose(self.dt, dr, rtol=1e-5, atol=1e-8))




# -*- coding: utf-8 -*-
# _zinc.py
# The zinc function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the zinc() function which calculates the magnitude
response of a cascade of comb filters.
"""

import numpy as np


def zinc(f, m=64, n=1):
    """Calculate the magnitude response of a cascade of ``n`` ``m``-th order comb filters.

    The magnitude of the filter response is calculated mathematically as:

    .. math::

        \\left|H(f)\\right| = \\left|\\frac{\\mathrm{sinc}(m f)}{\\mathrm{sinc}(f)}\\right|^n

    **Parameters:**

    f : ndarray
        The frequencies f at which the magnitude response is evaluated.

    m : int, optional
        The order of the comb filters.

    n : int, optional
        The number of comb filters in the cascade.

    **Returns:**

    HM : ndarray
        The magnitude of the frequency response of the cascade filter.

    """
    return np.fabs(np.sinc(m * f) / np.sinc(f)) ** n



# -*- coding: utf-8 -*-
# test_padr.py
# This module provides the tests for the padr function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the padr() function.
"""

import unittest
import numpy as np
from deltasigma import padr

class TestPadr(unittest.TestCase):
    """Test class for padr()"""
    def setUp(self):
        self.tv1 = np.eye(15)
        self.tv2 = np.arange(10)
        self.tv3 = np.array([])

    def test_padr_1(self):
        """Test function for padr() 1/3"""
        tr = padr(self.tv1, n=25, val=2)
        res = np.concatenate((self.tv1, 2.*np.ones((15, 10))), axis=1)
        self.assertTrue(np.allclose(tr, res, atol=1e-8, rtol=1e-5))

    def test_padr_2(self):
        """Test function for padr() 2/3"""
        # 1-d array
        tr = padr(self.tv2, n=25, val=1.5)
        res = np.hstack((self.tv2.reshape((1, -1)), 1.5*np.ones((1, 15))))
        self.assertTrue(np.allclose(tr, res, atol=1e-8, rtol=1e-5))

    def test_padr_3(self):
        """Test function for padr() 3/3"""
        # empty matrix array
        tr = padr(self.tv3, n=25, val=1.5)
        res = np.hstack((self.tv3.reshape((1, -1)), 1.5*np.ones((1, 25))))
        self.assertTrue(np.allclose(tr, res, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# _delay.py
# This module provides the delay function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the delay() function, used to delay a signal by a
specified amount of samples.
"""

import numpy as np


def delay(x, n=1):
    """Delay signal ``x`` by ``n`` samples.
    """
    x = np.atleast_1d(x)
    nx = max(x.shape)
    if nx <= n:
        y = np.zeros(x.shape)
    else:
        if len(x.shape) == 1:
            y = np.concatenate((np.zeros((n,)), x[:nx-n]))
        elif x.shape[0] > x.shape[1]:
            y = np.concatenate((np.zeros((n, x.shape[1])), x[:nx-n, :]), axis=0)
        else:
            y = np.concatenate((np.zeros((x.shape[0], n)), x[:, :nx-n]), axis=1)
    return y



# -*- coding: utf-8 -*-
# test_mapCtoD.py
# This module provides the tests for the mapCtoD function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the mapCtoD() function.
"""

from __future__ import division, print_function

import unittest
import numpy as np
import deltasigma as ds

from nose.tools import raises

class TestMapCtoD(unittest.TestCase):
    """Test class for mapCtoD()"""

    def setUp(self):
        pass

    def test_mapCtoD_1(self):
        """Test function for mapCtoD() 1/6"""
        # The first test comes straight from the DSToolbox doc
        # We map the standard continuous time DS modulator to its DT counterpart
        # and we check whether the TF is (1 - 1/z)**2
        LFc = (np.array([[0., 0.], [1., 0.]]),
               np.array([[1., -1.], [0., -1.5]]),
               np.array([[0., 1.]]),
               np.array(([[0., 0.]]))
              )
        tdac = [0, 1]
        LF, Gp = ds.mapCtoD(LFc, tdac)
        ABCD = np.vstack((np.hstack((LF[0], LF[1])),
                          np.hstack((LF[2], LF[3]))
                        ))
        H = ds.calculateTF(ABCD)
        self.assertTrue(np.allclose(H[0].num, [ 1., -2.,  1.], atol=1e-8, rtol=1e-5))
        self.assertTrue(np.allclose(H[0].den, [1., 0., 0.], atol=1e-8, rtol=1e-5))

    def test_mapCtoD_2(self):
        """Test function for mapCtoD() 2/6"""
        # zero delay NRZ test
        ABCDc = np.array([[0., 0.,  0., 0.04440879, -0.04440879],
                          [1., 0., -0.00578297, 0., -0.23997611],
                          [0., 1.,  0., 0., -0.67004646],
                          [0., 0.,  1., 0.,  0.]])
        tdac = np.array([[-1., -1], [0., 1.]])
        sys_d, Gp = ds.mapCtoD(ABCDc, tdac)
        ABCD = np.vstack((np.hstack((sys_d[0], sys_d[1])),
                          np.hstack((sys_d[2], sys_d[3]))
                        ))
        ABCDref = np.array([[1., 0., 0., 0.04440879, -0.04440879],
                            [0.99903645, 0.99710991, -0.0057774, 0.0221937, -0.26000208],
                            [0.49975909, 0.99903645,  0.99710991, 0.00739932, -0.79673041],
                            [0., 0., 1., 0., 0.]])
        self.assertTrue(np.allclose(ABCD, ABCDref, atol=1e-8, rtol=1e-5))

    def test_mapCtoD_3(self):
        """Test function for mapCtoD() 3/6"""
        # .1 delay NRZ
        ABCDc = np.array([[0., 0., 0., 0.0444, -0.0444],
                          [1., 0., -0.0058, 0., -0.2440],
                          [0., 1.,  0., 0., -0.6942],
                          [0., 0.,  1., 0., -0.0682]])

        tdac = np.array([[-1., -1], [0.1, 1.1]])
        sys_d, Gp = ds.mapCtoD(ABCDc, tdac)
        ABCD = np.vstack((np.hstack((sys_d[0], sys_d[1])),
                          np.hstack((sys_d[2], sys_d[3]))
                        ))
        ABCDref = np.array([[1., 0., 0., -0.0044, 0.0444, -0.0400],
                            [0.999, 0.9971, -0.0058, -0.0282, 0.0222, -0.2358],
                            [0.4998, 0.999, 0.9971, -0.0944, 0.0074, -0.7285],
                            [0., 0., 0., 0., 0., 1.],
                            [0., 0., 1., -0.0682, 0., 0.]])
        self.assertTrue(np.allclose(ABCD, ABCDref, atol=1e-4, rtol=1e-4))

    def test_mapCtoD_4(self):
        """Test function for mapCtoD() 4/6"""
        # Non-zero f0 test
        f0 = .2
        ABCDc = np.array([[0., 0., 1., -1.],
                          [1., 0., 0., -1.5],
                          [0., 1., 0., 0.]])
        tdac = np.array([0.1, 1.1])
        sys_d, Gp = ds.mapCtoD(ABCDc, tdac, f0=f0)
        ABCD = np.vstack((np.hstack((sys_d[0], sys_d[1])),
                          np.hstack((sys_d[2], sys_d[3]))
                        ))
        ABCDref = np.array([[1., 0., -0.1, 0.9355, -0.9],
                            [1., 1., -0.245, 0.4784, -1.7550],
                            [0., 0., 0., 0., 1.],
                            [0., 1., 0., 0., 0]])
        self.assertTrue(np.allclose(ABCD, ABCDref, atol=1e-4, rtol=1e-4))

    def test_mapCtoD_5(self):
        """Test function for mapCtoD() 5/6"""
        # Non-zero f0 test, short DAC pulse
        f0 = .2
        ABCDc = np.array([[0., 0., 1., -1.],
                          [1., 0., 0., -1.5],
                          [0, 1, 0, 0]])
        tdac = np.array([0.1, 0.4])
        sys_d, Gp = ds.mapCtoD(ABCDc, tdac, f0=f0)
        ABCD = np.vstack((np.hstack((sys_d[0], sys_d[1])),
                          np.hstack((sys_d[2], sys_d[3]))
                        ))
        print(ABCD)
        ABCDref = np.array([[1., 0, 0.9355, -0.3000],
                            [1., 1., 0.4784, -0.6750],
                            [0., 1., 0., 0.]])
        self.assertTrue(np.allclose(ABCD, ABCDref, atol=1e-4, rtol=1e-4))

    @raises(ValueError)
    def test_mapCtoD_6(self):
        """Test function for mapCtoD() 6/6"""
        # wrong ni
        f0 = 0.0
        ABCDc = np.array([[0., 0., 1., -1.],
                          [1., 0., 0., -1.5],
                          [0, 1, 0, 0]])
        tdac = np.array([0.1, 0.4, 0.1, 3.])
        sys_d, Gp = ds.mapCtoD(ABCDc, tdac, f0=f0)




# -*- coding: utf-8 -*-
# test_synthesizeNTF0.py
# This module provides the tests for the synthesizeNTF0 function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the synthesizeNTF0() function.
"""

import unittest
import numpy as np

from deltasigma._synthesizeNTF0 import synthesizeNTF0
from deltasigma._utils import cplxpair

class TestSynthesizeNTF0(unittest.TestCase):
    """Test class for synthesizeNTF0()"""

    def setUp(self):
        pass

    def test_synthesizeNTF0_1(self):
        """Test function for synthesizeNTF0() 1/8"""
        z, p, k = synthesizeNTF0(order=3, osr=64, opt=0, H_inf=1.5, f0=0.0)
        zref = [1., 1., 1.]
        pref = [.6694, .7654 + .2793j, .7654 - .2793j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_2(self):
        """Test function for synthesizeNTF0() 2/8"""
        # Up next: even order bandpass test
        z, p, k = synthesizeNTF0(order=4, osr=32, opt=0, H_inf=1.3, f0=.33)
        zref = [-0.4818 + 0.8763j, -0.4818 - 0.8763j, -0.4818 + 0.8763j,
                -0.4818 - 0.8763j]
        pref = [-0.5125 - 0.7018j, -0.5125 + 0.7018j, -0.3233 - 0.8240j,
                -0.3233 + 0.8240j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_3(self):
        """Test function for synthesizeNTF0() 3/8"""
        # repeat with zeros optimization
        z, p, k = synthesizeNTF0(order=3, osr=64, opt=1, H_inf=1.5, f0=0.0)
        zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
        pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_4(self):
        """Test function for synthesizeNTF0() 4/8"""
        # bandpass test
        z, p, k = synthesizeNTF0(order=4, osr=32, opt=1, H_inf=1.3, f0=.33)
        zref = [-0.4567 + 0.8896j, -0.4567 - 0.8896j, -0.5064 + 0.8623j,
                -0.5064 - 0.8623j]
        pref = [-0.5125 - 0.7014j, -0.5125 + 0.7014j, -0.3230 - 0.8239j,
                -0.3230 + 0.8239j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_5(self):
        """Test function for synthesizeNTF0() 5/8"""
        z, p, k = synthesizeNTF0(order=3, osr=64, opt=2, H_inf=1.5, f0=0.0)
        zref = [1.0000 + 0.0000j, 0.9993 + 0.0380j, 0.9993 - 0.0380j]
        pref = [0.7652 - 0.2795j, 0.7652 + 0.2795j, 0.6692 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_6(self):
        """Test function for synthesizeNTF0() 6/8"""
        z, p, k = synthesizeNTF0(order=4, osr=32, opt=2, H_inf=1.3, f0=.33)
        zref = [-0.4818 + 0.8763j, -0.4818 - 0.8763j, -0.4818 + 0.8763j,
                -0.4818 - 0.8763j]
        pref = [-0.5125 - 0.7018j, -0.5125 + 0.7018j, -0.3233 - 0.8240j,
                -0.3233 + 0.8240j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_7(self):
        """Test function for synthesizeNTF0() 7/8"""
        # zeros passed explicitly
        opt = [1.0000 + 0.0000j, 0.9986 + 0.06j, 0.9986 - 0.06j,
                0.9960 + 0.0892j, 0.9960 - 0.0892j]
        z, p, k = synthesizeNTF0(order=5, osr=32, opt=opt, H_inf=1.3, f0=0.0)
        zref = [1.0000 + 0.0000j, 0.9986 + 0.06j, 0.9986 - 0.06j,
                0.9960 + 0.0892j, 0.9960 - 0.0892j]
        pref = [0.8718 - 0.0840j, 0.8718 + 0.0840j, 0.9390 - 0.1475j,
                0.9390 + 0.1475j, 0.8491 + 0.0000j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeNTF0_8(self):
        """Test function for synthesizeNTF0() 8/8"""
        opt = 1
        z, p, k = synthesizeNTF0(order=4, osr=32, opt=opt, H_inf=1.3, f0=0.2)
        pref = [0.3742 - 0.7875j, 0.3742 + 0.7875j, 0.1616 - 0.8666j,
                0.1616 + 0.8666j]
        zref = [0.3358 - 0.9419j, 0.3358 + 0.9419j, 0.2819 - 0.9594j,
                0.2819 + 0.9594j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))




from __future__ import division
import unittest
import numpy as np
import deltasigma as ds
import scipy.io
import pkg_resources
from deltasigma._utils import mround


class TestBplogsmooth(unittest.TestCase):
    """Test class for bplogsmooth()"""

    def setUp(self):
        fname = pkg_resources.resource_filename(
            __name__, "test_data/test_bplogsmooth.mat")
        self.data = scipy.io.loadmat(fname)
        f0 = 1./8
        OSR = 64
        order = 8
        N = 8192
        H = ds.synthesizeNTF(order, OSR, 1, 1.5, f0)
        fB = int(np.ceil(N/(2. * OSR)))
        ftest = int(mround(f0*N + 1./3*fB))
        u = 0.5*np.sin(2*np.pi*ftest/N*np.arange(N))
        v, xn, xmax, y = ds.simulateDSM(u, H)
        spec = np.fft.fft(v*ds.ds_hann(N))/(N/4)
        X = spec[:N/2 + 1]
        self.f, self.p = ds.bplogsmooth(X, ftest, f0)

    def test_one(self):
        """Test function 1/2 for bplogsmooth()"""
        data = self.data
        f = self.f
        self.assertTrue(np.allclose(f, data['f'], atol=1e-9, rtol=1e-5))

    def test_two(self):
        """Test function 2/2 for bplogsmooth()"""
        data = self.data
        p = self.p
        self.assertTrue(np.allclose(p, data['p'], atol=1e-9, rtol=1e-5))
